/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: PMDCollectionManager.java,v 1.3 2003/06/09 06:28:39 tanderson Exp $
 *
 * Date         Author  Changes
 * 03/07/2000   jima    Created
 * 04/04/2000   jima    Added createTreeMap and createTreeSet
 */
package org.exolab.core.database.recman;


import java.util.Comparator;
import org.exolab.core.foundation.CollectionManagerIfc;
import org.exolab.core.foundation.FailedToCreateCollectionException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.exolab.core.foundation.HashMapIfc;
import org.exolab.core.foundation.VectorIfc;
import org.exolab.core.foundation.TreeSetIfc;
import org.exolab.core.foundation.TreeMapIfc;

/**
 * This is an implementation of the collection manager specific to the page
 * managed database. These are collections that are persistent capable.
 * <p>
 * The PMDCollectionManager is a singleton classes and is accessible through
 * PMDCollectionManager.instance
 *
 * @version     $Revision: 1.3 $ $Date: 2003/06/09 06:28:39 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         org.exolab.core.foundation.CollectionManagerIfc
 **/
public class PMDCollectionManager 
    implements CollectionManagerIfc
{

    /**
     *  The logger
     */
    private static final Log _log = LogFactory.getLog(PMDCollectionManager.class);

    /**
     * Create an instance of this collection manager. The constructor is scoped
     * private so that it can only be creaed within this class.
     */
    private PMDCollectionManager()
    {
    }

    /**
     * Return the singleton reference to the collection manager. If one does 
     * not exist, create it before returning it.
     *
     * @return      PMDCollectionManager
     */
    public static PMDCollectionManager instance()
    {
        if (instance__ == null)
        {
            instance__ = new PMDCollectionManager();
        }

        return instance__;
    }

    // implementation of CollectionManagerIfc.reateVector()
    public VectorIfc createVector()
        throws FailedToCreateCollectionException
    {
        VectorIfc result = null;

        try
        {
            result = new PMDVector();
        }
        catch (Exception exception)
        {
            _log.error(exception);
            throw new FailedToCreateCollectionException(
                "failed to create a vector : - " + exception);
        }

        return result;
    }

    // implementation of CollectionManagerIfc.createHashMap
    public HashMapIfc createHashMap()
        throws FailedToCreateCollectionException
    {
        HashMapIfc result = null;

        try
        {
            result = new PMDHashMap();
        }
        catch (Exception exception)
        {
            _log.error(exception);
            throw new FailedToCreateCollectionException(
                "failed to create a hashmap : - " + exception);
        }

        return result;
    }

    // implementation of CollectionManagerIfc.createTreeMap
    public TreeMapIfc createTreeMap()
        throws FailedToCreateCollectionException
    {
        TreeMapIfc result = null;

        try
        {
            result = new PMDTreeMap();
        }
        catch (Exception exception)
        {
            _log.error(exception);
            throw new FailedToCreateCollectionException(
                "failed to create a TreeMap : - " + exception);
        }

        return result;
    }

    // implementation of CollectionManagerIfc.createTreeMap(Comparator)
    public TreeMapIfc createTreeMap(Comparator comparator)
        throws FailedToCreateCollectionException
    {
        TreeMapIfc result = null;

        try
        {
            result = new PMDTreeMap(comparator);
        }
        catch (Exception exception)
        {
            _log.error(exception);
            throw new FailedToCreateCollectionException(
                "failed to create a TreeMap : - " + exception);
        }

        return result;
    }

    // implementation of CollectionManagerIfc.createTreeSet
    public TreeSetIfc createTreeSet()
        throws FailedToCreateCollectionException
    {
        TreeSetIfc result = null;

        try
        {
            result = new PMDTreeSet();
        }
        catch (Exception exception)
        {
            _log.error(exception);
            throw new FailedToCreateCollectionException(
                "failed to create a tree set : - " + exception);
        }

        return result;
    }

    // implementation of CollectionManagerIfc.createTreeSet(Comparator)
    public TreeSetIfc createTreeSet(Comparator comparator)
        throws FailedToCreateCollectionException
    {
        TreeSetIfc result = null;

        try
        {
            result = new PMDTreeSet(comparator);
        }
        catch (Exception exception)
        {
            _log.error(exception);
            throw new FailedToCreateCollectionException(
                "failed to create a TreeSet : - " + exception);
        }

        return result;
    }


    private static PMDCollectionManager instance__ = null;
}



