/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: PMDTreeSet.java,v 1.2 2001/06/12 07:11:35 jima Exp $
 *
 * Date         Author  Changes
 * 04/05/2000   jima    Created
 */
package org.exolab.core.database.recman;


import java.util.Enumeration;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;
import java.util.Comparator;
import java.util.TreeSet;
import java.io.Serializable;
import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.Hashtable;
import java.util.Enumeration;
import org.exolab.core.foundation.PersistentObject;
import org.exolab.core.foundation.TreeSetIfc;

/**
 * This class implements the TreeSetIfc for the PMD database. The current 
 * implementation simply wraps the java.util.TreeSet and makes it persistent.
 * <p>
 * Future implementations will store handles instead of objects and hence 
 * making it more concurrent aware.
 *
 * @version     $Revision: 1.2 $ $Date: 2001/06/12 07:11:35 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         org.exolab.core.foundation.TreeSetIfc
 * @see         org.exolab.core.foundation.PersistentObject
 **/
public class PMDTreeSet
    extends PersistentObject
    implements TreeSetIfc, Externalizable
{
    /**
     * The default constructor initialises the the TreeSet object.
     */
    public PMDTreeSet()
    {
        set_ = new TreeSet();
    }
    
    /**
     * Construct a set with the specified constructor. The constructor is used
     * to compare all elements in the set
     *
     * @param       comparator          used for ordering
     */
    public PMDTreeSet(Comparator comparator)
    {
        set_ = new TreeSet(comparator);
    }
    
    /**
     * Returns an iterator for the elements in this set. The elements
     * are returned in ascending order.
     *
     * @return      Iterator
     */
    public Iterator iterator()
    {
        return set_.iterator();
    }

    /**
     * Returns the number elements in the set
     *
     * @return                  number of mappings in this set
     */
    public int size()
    {
        return set_.size();
    }


    /**
     * Test is the set is empty
     *
     * @return      boolean     true if the set is empty and false otherwise
     */
	public boolean isEmpty()
    {
        return set_.isEmpty();
    }

    /**
     * Test whether the set contains the speciied object.If the object cannot
     * be compared with other elements in the set the ClassCastException is 
     * raised.
     *
     * @param       object          object to test for
     * @return      boolean         true if present; false otherwise
     * @throws      ClassCastException
     */
    public boolean contains(Object object)
    {
        return set_.contains(object);
    }

    /**
     * Adds the specified element to this set if it is not already present.If
     * the element alsready exists then return false, otherwise return true. If
     * the specified object cannot be compared with other objects in the set it 
     * will throw the ClassCastException
     *
     * @param       object          object to add
     * @return      true            if it was added and false otherwise
     * @throws      ClassCastException
     */
    public boolean add(Object object)
    {
        return set_.add(object);
    }

    /**
     * Removes the given element from this set if it is present. If the set
     * contained the object return true otherwise return false. If the object
     * cannot be compared with other elements in the set then throw the
     * ClassCastException
     *
     * @param       object          object to remove
     * @return      boolean         true if the object originally existed
     * @exception   ClassCastException
     */
    public boolean remove(Object object)
    {
        return set_.remove(object);
    }


    /**
     * Removes all of the elements from this set.
     */
    public void clear()
    {
        set_.clear();
    }

    /**
     * Returns the comparator used to order this set or null if this set uses
     * natural ordering
     *
     * @return      Comparator
     */
    public Comparator comparator()
    {
        return set_.comparator();
    }

    /**
     * Returns the first (lowest) key currently in this sorted set.It throws 
     * NoSuchElementException if the element does not exist.
     *
     * @return      Object
     * @throws      NoSuchElementException 
     */
    public Object first()
    {
        return set_.first();
    }

    /**
     * Returns the last (highest) key currently in this sorted map.It throws 
     * NoSuchElementException if the element does not exist.
     *
     * @return      Object
     * @throws      NoSuchElementException 
     */
    public Object last()
    {
        return set_.last();
    }


    // implementation of Externalizable.writeExternal
    public void writeExternal(ObjectOutput stream)
        throws IOException
    {
        stream.writeLong(serialVerionUID);
        stream.writeObject(set_);
        super.writeExternal(stream);
    }

    // implementation of Externalizable.writeExternal
    public void readExternal(ObjectInput stream)
        throws IOException, ClassNotFoundException
    {
		long version = stream.readLong();
		if (version == serialVerionUID) {	
            set_ = (TreeSet)stream.readObject();
            super.readExternal(stream); 
        } else {
            throw new IOException("PMDTreeSet with version " +
                version + " is not supported.");
        }
    }

    static final long serialVerionUID = 1;
    private TreeSet set_ = null;
}


