/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: PMDVector.java,v 1.5 2001/06/12 07:11:35 jima Exp $
 *
 * Date         Author  Changes
 * 2/23/2000    jima    Created
 */
package org.exolab.core.database.recman;


import java.util.Enumeration;
import java.util.Vector;
import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.Hashtable;
import java.util.Enumeration;
import org.exolab.core.foundation.VectorIfc;
import org.exolab.core.foundation.PersistentCapableIfc;
import org.exolab.core.foundation.PersistentObject;

/**
 * This concrete class implements the VectorIfc interface. and defines a
 * PagemManagedDatabase specific implementation for this type of database.
 * This Vector is specific to objects of type PersistentCapableIfc
 *
 * @version     $Revision: 1.5 $ $Date: 2001/06/12 07:11:35 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/
public class PMDVector
    extends PersistentObject
    implements VectorIfc, Externalizable
{
    /**
     * Construct an instance of this persistent capable class. Once this has
     * been constructed then it could be made persistent
     */
    public PMDVector()
    {
        vector_ = new Vector(50, 10);
    }

    /**
     * Return the size of the vector
     *
     * @return      int
     */
    public int size()
    {
        return vector_.size();
    }

    /**
     * Check to see of the vector is empty.
     *
     * @return      boolean     true if it is empty and false otherwise
     */
    public boolean isEmpty()
    {
        return vector_.isEmpty();
    }

    /**
     * Return an enumeration of all elements in the Vector
     *
     * @return      Enumeration
     */ 
    public Enumeration elements()
    {
        return vector_.elements();
    }

    /** 
     * Test to see whether the specified object exists. This is a very
     * expensive operation since it does a linear search until a match
     * is found.
     *
     * @param       element     element to test for
     * @return      boolean     true if it is present and false otherwise
     */
    public synchronized boolean contains(PersistentCapableIfc element)
    {
        return vector_.contains(element);
    }

    /**
     * Return the index of the specified element or -1 if it does not exist.
     *
     * @param       element     element to search for
     * @return      int         index of the found element or -1.
     */
    public synchronized int indexOf(PersistentCapableIfc element)
    {
        return vector_.indexOf(element);
    }

    /**
     * Return the object at the specified index or null if one does not
     * exist
     *
     * @param       index       index to query on
     * @return      PersistentCapableIfc
     */
    public synchronized PersistentCapableIfc elementAt(int index)
    {
        return (PersistentCapableIfc)vector_.elementAt(index);
    }

    /**
     * Return the first element in the Vector or null if it is empty
     *
     * @return      PersistentCapableIfc
     */
    public synchronized PersistentCapableIfc firstElement()
    {
        return this.elementAt(0);
    }

    /**
     * Return the last element in the Vector or null if it is empty
     *
     * @return      PersistentCapableIfc
     */
    public synchronized PersistentCapableIfc lastElement()
    {
        return this.elementAt(size() - 1);
    }

    /**
     * Remove the element at the specified index
     *
     * @param       index       index of element to remove
     */
    public synchronized void removeElementAt(int index)
    {
        vector_.removeElementAt(index);
    }

    /**
     * Add the specified element to the end of the vector. Although this method
     * does specify the element to be of type Object a further constraint is 
     * placed on this argument in that it must also be of type PersistentObject
     * If this second prerequisite is not fullfilled then the runtime error 
     * UnsupportedObjectException, which indicates that the supplied object 
     * could not be added to this vector.
     *
     * @param       element     element to add
     */
    public synchronized void addElement(PersistentCapableIfc element)
    {
        vector_.addElement(element);
    }

    /**
     * Remove the specified element from the Vector. This is an expensive 
     * operation since it performs a linear search to determine if the object
     * exists prior to removinf it 
     *
     * @param       element     element to remove
     * @return      boolean     true if it was removed
     */
    public synchronized boolean removeElement(PersistentCapableIfc element)
    {
        return vector_.removeElement(element);
    }

    /**
     * Remove all the elements from the Vector. Functionally, equivalent to 
     * the clear method
     */
    public synchronized void removeAllElements()
    {
        vector_.removeAllElements();
    }

    /**
     * Return the objects in the Vector as an array.
     *
     * @return      Object[]
     */
    public synchronized Object[] toArray()
    {
        return vector_.toArray();
    }

    /** 
     * Return the object at the specified index or null
     *
     * @return      PersistentCapableIfc
     */
    public synchronized PersistentCapableIfc get(int index)
    {
        return this.elementAt(index);
    }

    /** 
     * Add the specified element to the end of the Vector. This subclass
     * adds a further constraint on the incoming parameter which must be
     * of type PeristentCapableIfc
     *
     * @param       element     element to add
     * @return      boolean     true if successful and false otherwise
     */
    public synchronized boolean add(PersistentCapableIfc element)
    {
        return vector_.add(element);
    }

    /**
     * Remove the specified element from the Vector
     *
     * @param       element     element to remove
     * @return      boolean     true if successful; and false otherwise 
     */
    public synchronized boolean remove(PersistentCapableIfc element)
    {
        return this.removeElement(element);
    }

    /**
     * Add the element at the specified index. An additional constraint applies
     * to element in that it must also be of type PersistentCapableIfc
     *
     * @param       index       index 
     * @param       element     element to add
     */
    public synchronized void add(int index, PersistentCapableIfc element)
    {
        vector_.add(index, element);
    }

	/**
	 * Set the object at the location given, overwriting the existing object 
	 * if it exists. An additional constraint applies
     * to element in that it must also be of type PersistentCapableIfc
	 *
     * @param       index       index 
     * @param       element     element to add
	 *
     */
	public synchronized void set(int index, PersistentCapableIfc element)
    {
        vector_.set(index, element);
    }

    /**
     * Remove the element at the specified index and return it to the 
     * client
     *
     * @param       index       index to query
     * @return      PersistentCapableIfc
     */
    public synchronized PersistentCapableIfc remove(int index)
    {
        return (PersistentCapableIfc)vector_.remove(index);
    }

    /**
     * Clear the vector
     */
    public synchronized void clear()
    {
        vector_.clear();
    }


    // implementation of Externalizable.writeExternal
    public void writeExternal(ObjectOutput stream)
        throws IOException
    {
        stream.writeLong(serialVerionUID);
        stream.writeObject(vector_);
        super.writeExternal(stream);
    }

    // implementation of Externalizable.writeExternal
    public void readExternal(ObjectInput stream)
        throws IOException, ClassNotFoundException
    {
		long version = stream.readLong();
		if (version == serialVerionUID) {	
            vector_ = (Vector)stream.readObject();
            super.readExternal(stream); 
        } else {
            throw new IOException("PMDVector with version " +
                version + " is not supported.");
        }
    }

    static final long serialVerionUID = 1;
    private Vector vector_ = null;
}


