/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: CollectionManagerIfc.java,v 1.2 2000/04/04 22:52:26 jima Exp $
 *
 * Date         Author  Changes
 * 03/05/2000   jima    Created
 * 04/04/2000   jima    Added createTreeSet and createTreeMap to the 
 *                      interface
 */
package org.exolab.core.foundation;


import java.util.Comparator;

/**
 * This interface defines methods for creating specific types of collections
 * including hashtables and vectors. It is part of the pesistence framework.
 *
 * @version     $Revision: 1.2 $ $Date: 2000/04/04 22:52:26 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/
public interface CollectionManagerIfc
{
    /**
     * Create a vector with default capacity and return a reference to the 
     * to the client. If the vector cannot be created then throw FailedToCreate-
     * CollectionException exception.
     *
     * @return      VectorIfc       returned object
     * @exception   FailedToCreateCollectionException
     */
    public VectorIfc createVector()
        throws FailedToCreateCollectionException;

    /**
     * Create a hashmap with default capacity and return a reference to the 
     * client. If the hahsmap cannot be created then throw FailedToCreate-
     * CollectionException exception.
     *
     * @return      HashMapIfc      returned object
     * @exception   FailedToCreateCollectionException
     */
    public HashMapIfc createHashMap()
        throws FailedToCreateCollectionException;

    /**
     * Create a treemap with default capacity and return a reference to the 
     * client. If the treemap cannot be created then throw FailedToCreate-
     * CollectionException exception
     *
     * @return      TreeMapIfc      returned object
     * @exception   FailedToCreateCollectionException
     */
    public TreeMapIfc createTreeMap()
        throws FailedToCreateCollectionException;

    /**
     * Create a treemap with the specified comparator and return a reference 
     * client. If the treemap cannot be created then throw FailedToCreate-
     * CollectionException exception
     *
     * @param       comparator      specified comparator
     * @return      TreeMapIfc      returned object
     * @exception   FailedToCreateCollectionException
     */
    public TreeMapIfc createTreeMap(Comparator comparator)
        throws FailedToCreateCollectionException;

    /**
     * Create a treeset with default capacity and return a reference to the 
     * client. If the treeset cannot be created then throw FailedToCreate-
     * CollectionException exception
     *
     * @return      TreeSetIfc      returned object
     * @exception   FailedToCreateCollectionException
     */
    public TreeSetIfc createTreeSet()
        throws FailedToCreateCollectionException;

    /**
     * Create a treeset with the specified comparator and return a reference 
     * client. If the treeset cannot be created then throw FailedToCreate-
     * CollectionException exception
     *
     * @param       comparator      specified comparator
     * @return      TreeSetIfc      returned object
     * @exception   FailedToCreateCollectionException
     */
    public TreeSetIfc createTreeSet(Comparator comparator)
        throws FailedToCreateCollectionException;
}


