/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: DatabaseManagerIfc.java,v 1.1 2000/02/22 07:37:55 jima Exp $
 *
 * Date         Author  Changes
 * 1/13/2000    jima    Created
 */
package org.exolab.core.foundation;

import java.util.Properties;

/**
 * The DatabaseManager interface defines the creation point for the database. It
 * simply has one method, open, which returns a reference to a  DatabaseIfc 
 * instance
 *
 * @version     $Revision: 1.1 $ $Date: 2000/02/22 07:37:55 $
 * @author      <a href="mailto:jima@exoffice.org">Jim Alateras</a>
 **/
public interface DatabaseManagerIfc
{
    /**
     * Open a Database with the specified name. If the database does not exist
     * then attemtp to create it before opening it.If it cannot do either then
     * throw the FailedToCreateDatabase exception
     * <p>
     * The input parameter 'props' holds vendor specific properties which
     * can be used during the creation. The clietn can pass a null props param.
     *
     * @param       name            name of the database
     * @param       props           vendor specific properties
     * @return      Databaseifc     created or opened database
     * @exception   FailedToCreateDatabaseException
     */
    public DatabaseIfc create(String name, Properties props)
        throws FailedToCreateDatabaseException;
}
