/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: FailedToReleaseLocksException.java,v 1.1 2000/02/22 07:37:55 jima Exp $
 *
 * Date         Author  Changes
 * 1/13/2000    jima    Created
 */
package org.exolab.core.foundation;


import java.util.Vector;
import java.util.Enumeration;

/**
 * This excpetion is thrown whenever the lock service fails to release one or
 * more locks. It manages the set of locks that it failed to release.
 *
 * @version     $Revision: 1.1 $ $Date: 2000/02/22 07:37:55 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/
public class FailedToReleaseLocksException 
    extends Exception
{
    /**
     * Default constructor with no additional info.
     */
    public FailedToReleaseLocksException()
    {
        super();
    }

    /**
     * Construct an instance of this exception with the specified reason.
     *
     * @param       reason
     */
    public FailedToReleaseLocksException(String reason)
    {
        super(reason);
    }

    /**
     * Add a reference to the PersistentObject object whose lock the service
     * could not release
     *
     * @param       obj         locked object
     */
    public void addObject(PersistentObject obj)
    {
        failed_.addElement(obj);
    }

    /**
     * Retrieve an enumeration of the failed PesistentObject objects ehose
     * locks were not released
     */
    public Enumeration getFailedObjects()
    {
        return failed_.elements();
    }

    /**
     * Return a string representation of this exception
     *
     * @return      String
     */
    public String toString()
    {
        StringBuffer buf = new StringBuffer();
        Enumeration iter = getFailedObjects();

        while (iter.hasMoreElements())
        {
            PersistentObject obj = (PersistentObject)iter.nextElement();
            buf.append(obj.toString());
        }

        return buf.toString();
    }


    private Vector failed_ = new Vector();
}

