/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: HashMapIfc.java,v 1.3 2000/04/28 07:37:51 jima Exp $
 *
 * Date         Author  Changes
 * 2/23/2000    jima    Created
 * 3/05/2000    jima    Constrained the interface to apply to
 */
package org.exolab.core.foundation;


import java.util.Enumeration;
import java.io.Serializable;

/**
 * This interface abstracts the Persistent HashMap data structure so that it
 * can be mapped to various persistent and transient datastores. It basically
 * uses the JDK1.1.2 java.util.Hashtable to derive the interface but applies to
 * keys and objects that support the PersistentCapableIfc type.
 *
 * @version     $Revision: 1.3 $ $Date: 2000/04/28 07:37:51 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         org.exolab.core.foundation.PersistentCapableIfc
 * @see         java.io.Serializable
 **/
public interface HashMapIfc
    extends PersistentCapableIfc, Serializable
{
    /**
     * Return the size of the map
     *
     * @return      int
     */
    public int size();

    /**
     * Test for an empty hashmap
     *
     * @return      boolean     true if empty; false otherwise
     */
    public boolean isEmpty();

    /**
     * Return the enumeration of all keys in the map
     *
     * @return      Enumeration
     */ 
    public Enumeration keys();

    /**
     * Return an enumeration of all the elements in the map
     *
     * @return      Enumeration
     */
    public Enumeration elements();

    /** 
     * Test to see whether the map contains the a key for this value
     *
     * @param       element     element to test against
     * @return      boolean     true if key is mapped; false otherwise
     */
    public boolean containsValue(PersistentCapableIfc element);
    
    /**
     * Return true if the specified key maps to an element in the map
     *
     * @param       key         key to test against
     * @return      boolean     true if it maps; false otherwise
     */
    public boolean containsKey(Object key);

    /**
     * Return the object mapped to this key.
     *
     * @param       key                     key to lookup
     * @return      PersistentCapableIfc    mapped object or null if it is not 
     *                                      mapped
     */
    public PersistentCapableIfc get(Object key);

    /**
     * Add the key and associated element to the map and return a reference
     * to the object
     *
     * @param       key                     key to map
     * @param       element                 corresponding element
     * @return      PersistentCapableIfc    the object added to the map.
     */
    public PersistentCapableIfc put(Object key, PersistentCapableIfc element);

    /**
     * Remove the object mapped to the specified key from the map and return
     * it to the client
     *
     * @param       key                     key to remove
     * @param       PersistentCapableIfc    coreesponding element
     */
    public PersistentCapableIfc remove(Object key);

    /**
     * Clear the map
     */
    public void clear();

    /**
     * Return a refernece to the enclosed map. To be used it will have
     * be cast to the Hashtable
     *
     * @return      Object
     */
    public Object getMapObject();
}

