/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: LockServiceIfc.java,v 1.1 2000/02/22 07:37:55 jima Exp $
 *
 * Date         Author  Changes
 * 1/13/2000    jima    Created
 */
package org.exolab.core.foundation;



import java.util.Enumeration;

/**
 * Thew lock service is responsible for manmaging object level locking of 
 * PersistentObject instances.  
 *
 * @version     $Revision: 1.1 $ $Date: 2000/02/22 07:37:55 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/
public interface LockServiceIfc
{
    /**
     * Acquire a read lock on the specified PersistentObject. It will wait
     * DefaultReadLockWaitTime ms, which is configurable. If the lock cannot be
     * acquird wihin the specified time the function will throw 
     * FailedToAcquireLockException
     *
     * @param       obj         object to read-lock
     * @excepption  FailedToAcquireLockException
     */
    public void acquireReadLock(SessionIfc session, PersistentObject obj)
        throws FailedToAcquireLockException;

    /**
     * Acquire a write lock on the specified PersistentObject. It will wait
     * DefaultWriteLockWaitTime ms which is configurable. If the lock has not
     * been acquired, within the specified time, the function will throw 
     *
     * @param       obj         object to write-lock
     * @exception   FailedToAcqwuireLockException
     */
    public boolean aquireWriteLock(SessionIfc session, PersistentObject obj)
        throws FailedToAcquireLockException;

    /**
     * Acquire an exclusive lock on the specified object. It will wait for
     * DefaultExclusiveLockWaitTime ms, whihc is configurable.An exclusive
     * lock cannot be shared with other users, as read andwrite locks. If the
     * lock cannot be acquired within the specified time then throw the 
     * FailedToAcquireLockException
     *
     * @param       obj         object to exclusively-lock
     * @exception   FailedToAcquireLockException         
     */
    public void acquireExclusiveLock(SessionIfc session, PersistentObject obj)
        throws FailedToAcquireLockException;

    /**
     * Acquire a read lock on the specified PersistentObject. If the lock 
     * cannot be acquird wihin 'wait' ms the function will throw 
     * FailedToAcquireLockException
     *
     * @param       obj         object to read-lock
     * @param       wait        time to wait in ms
     * @excepption  FailedToAcquireLockException
     */
    public void acquireReadLock(SessionIfc session, PersistentObject obj, long wait)
        throws FailedToAcquireLockException;

    /**
     * Acquire a write lock on the specified PersistentObject. If the lock 
     * cannot be acquired, within 'wait' ms the function will throw
     * FailedToAcquireLockException 
     *
     * @param       obj         object to write-lock
     * @param       wait        time to wait in ms
     * @exception   FailedToAcquireLockException
     */
    public boolean aquireWriteLock(SessionIfc session, PersistentObject obj, long wait)
        throws FailedToAcquireLockException;

    /**
     * Acquire an exclusive lock on the specified object. An exclusive lock 
     * cannot be shared with other users, as read and write locks. If the
     * lock cannot be acquired within 'wait' ms then throw the 
     * FailedToAcquireLockException
     *
     * @param       obj         object to exclusively-lock
     * @param       wait        time to wait in ms
     * @exception   FailedToAcquireLockException         
     */
    public void acquireExclusiveLock(SessionIfc session, PersistentObject obj, long wait)
        throws FailedToAcquireLockException;

    /**
     * Acquire read locks on all the specified PersistentObject(s). It will 
     * wait DefaultReadLockWaitTime ms, which is configurable. If <b>all</b>the 
     * locks cannot be acquird wihin the specified time the function will throw 
     * FailedToAcquireLocksException. This exception defines the list of locks
     * that it failed to acquire
     *
     * @param       obj         collection of PersistentObject instances
     * @excepption  FailedToAcquireLocksException
     */
    public void acquireReadLocks(SessionIfc session, Enumeration objs)
        throws FailedToAcquireLocksException;

    /**
     * Acquire write locks on all the specified PersistentObject(s). It will 
     * wait DefaultWriteLockWaitTime ms which is configurable. If <b>all</b>
     * the lock cannot be acquired, within the specified time, the function 
     * will throw FailedToAcquireLocksException
     *
     * @param       objs        collection of PersistentObject instances
     * @exception   FailedToAcquireLocksException
     */
    public boolean aquireWriteLocks(SessionIfc session, Enumeration objs)
        throws FailedToAcquireLocksException;

    /**
     * Acquire exclusive locks on all the specified PersistentObject(s). It 
     * will wait for DefaultExclusiveLockWaitTime ms, which is configurable.
     * An exclusive lock cannot be shared with other users, as read and write
     * locks. If <b>all</b> the locks cannot be acquired within the specified 
     * time then throw the FailedToAcquireLocksException
     *
     * @param       objs        collection of PersistentObject instances
     * @exception   FailedToAcquireLockException         
     */
    public void acquireExclusiveLocks(SessionIfc session, Enumeration objs)
        throws FailedToAcquireLocksException;

    /**
     * Acquire read locks on the specified PersistentObject(s). If <b>all</b>the
     * lock  cannot be acquird wihin 'wait' ms the function will throw 
     * FailedToAcquireLocksException
     *
     * @param       objs        collection of PersistentObject instances
     * @param       wait        time to wait in ms
     * @excepption  FailedToAcquireLocksException
     */
    public void acquireReadLocks(SessionIfc session, Enumeration objs, long wait)
        throws FailedToAcquireLocksException;

    /**
     * Acquire write locks on the specified PersistentObject(s). If <b>all</b>
     * the lock cannot be acquired, within 'wait' ms the function will throw
     * FailedToAcquireLocskException 
     *
     * @param       objs         collection of PersistentObject instances
     * @param       wait        time to wait in ms
     * @exception   FailedToAcquireLocksException
     */
    public boolean aquireWriteLocks(SessionIfc session, Enumeration objs, long wait)
        throws FailedToAcquireLocksException;

    /**
     * Acquire exclusive locks on the specified objects. An exclusive lock 
     * cannot be shared with other users, as read and write locks. If <b>all</b>
     * the lock cannot be acquired within 'wait' ms then throw the 
     * FailedToAcquireLocksException
     *
     * @param       objs        collection of PersistentObject instances
     * @param       wait        time to wait in ms
     * @exception   FailedToAcquireLocksException         
     */
    public void acquireExclusiveLocks(SessionIfc session, Enumeration objs, long wait)
        throws FailedToAcquireLocksException;

    /**
     * Release the lock on the specified object. If the objects is not
     * locked than throw FailedToReleaseLockException
     *
     * @param       obj         object to release
     * @exception   FailedToReleaseLockException
     */
    public void releaseLock(PersistentObject obj)
        throws FailedToReleaseLockException;

    /**
     * Release all the locks specified by objs. If one or more of the locks
     * are not available, or cannot be locked then throw the 
     * FailedToReleaseLocksException. This will contain a list of PersistentObj-
     * ects that with no locks.
     *
     * @param       objs        objects to unlock
     * @exception   FailedToReleaseLocksException
     */
    public void releaseLocks(Enumeration objs)
        throws FailedToReleaseLocksException; 

    /**
     * Release all the locks associated with this session. If it encounters a 
     * problem releasing one or more locks then throw 
     * FailedToReleaseLocksException.
     *
     * @exception   FailedToReleaseLocksException
     */
    public void releaseLocks(SessionIfc session)
        throws FailedToReleaseLocksException;

    /**
     * Check whether the specified object is locked
     *
     * @param       obj         object to check
     * @return      boolean     true if locked and false otherwise
     */
    public boolean isLocked(PersistentObject obj);

    /**
     * Return the set of objectslocked for the specified session
     *
     * @param       session     session to query
     * @return      Enumeration
     */
    public Enumeration getLockedObjects(SessionIfc session);

    /**
     * Return an enumeration of all locked object.
     *
     * @return      Enumeration
     */
    public Enumeration getAllLockedObjects();

    /**
     * Release all the locks maintained by the service. If it encounters a 
     * problem releasing one or more locks then throw 
     * FailedToReleaseLocksException.
     *
     * @exception   FailedToReleaseLocksException
     */
    public void releaseAllLocks();
}       
