/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: ObjectVersion.java,v 1.1 2000/02/22 07:37:56 jima Exp $
 *
 * Date         Author  Changes
 * 1/14/2000    jima    Created
 */
package org.exolab.core.foundation;

import java.io.Serializable;

/**
 * This class defines an object version. Two objects with the same object id
 * but different version numbers are different object instances. The datastore
 * is responsible for managing the object version.
 *
 * @version     $Revision: 1.1 $ $Date: 2000/02/22 07:37:56 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         PersistentObject
 **/
public class ObjectVersion
    implements Serializable
{
    /**
     * The default constructore is used to instantiate an instance of this
     * class without initializing the id attribute
     */
    public ObjectVersion()
    {
    }

    /**
     * Construct an object with the specified version
     *
     * @param       version         object version
     */
    public ObjectVersion(long version)
    {
        version_ = version;
    }

    /**
     * Set the persistent object version number
     *
     * @param       version
     */
    public void setVersion(long version)
    {
        version_ = version;
    }

    /**
     * Return the object's version number
     *
     * @return      long
     */
    public long getVersion()
    {
        return version_;
    }

    /**
     * Determine whether the two objects are equality. Equality in this 
     * domain implies that the two version numbers are identical. 
     *
     * @param       object      the object to test against
     * @return      boolean     return true if it is equals and false otherwise
     */
    public boolean equals(Object object)
    {
        boolean result = false;

        if (object instanceof ObjectVersion)
        {
            if (version_ == ((ObjectVersion)object).version_)
            {
                result = true;
            }
        }

        return result;
    }

    /**
     * Return a string representation of this object. 
     *
     * @string
     */
    public String toString()
    {
        return "version_ = " + version_;
    }


    private long version_ = 0;
}
