/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: PersistentCapableIfc.java,v 1.2 2000/05/19 00:39:54 jima Exp $
 *
 * Date         Author  Changes
 * 1/26/2000    jima    Created
 */
package org.exolab.core.foundation;

/**
 * This interface is used to mark persistent capable classes. These classes can
 * be made persistent but do not necessarily have to be persistent since they can
 * also exist in transient store. 
 * <p>
 * Currently this class does constrain the definitions of ObjectId and 
 * ObjectVersion but in future these maybe changed to interfaces, hence giving
 * the implementor additional freedom.
 *
 * @version     $Revision: 1.2 $ $Date: 2000/05/19 00:39:54 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         org.exolab.core.foundation.ObjectId
 * @see         org.exolab.core.foundation.ObjectVersion
 **/
public interface PersistentCapableIfc
{
    /**
     * Return the object identifier associated with this object
     *
     * @return      ObjectId
     */
    public ObjectId getObjectId();

    /**
     * Return the object version associated with this object
     *
     * @return      ObjectId
     */
    public ObjectVersion getObjectVersion();

    /**
     * Return the minimum size of this object. This vould be used to 
     * preallocate the object size for more efficient storage.
     *
     * @return      int
     */
    public int getMinimumObjectSize();

    /**
     * Set the minimum size that the object should occurpy. It is only
     * a hint.
     *
     * @param       size        minimum object size
     */
    public void setMinimumObjectSize(int size);
}

