/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: SessionManagerIfc.java,v 1.2 2000/03/09 23:52:43 jima Exp $
 *
 * Date         Author  Changes
 * 1/14/2000    jima    Created
 */

package org.exolab.core.foundation;

import java.util.Enumeration;


/**
 * This interface is responsible for managing all aspects of a sessions 
 * lifecycle from creation, administering and  destruction. A session is
 * single-threaded entity and hence one session per thread. 
 *
 * @version     $Revision: 1.2 $ $Date: 2000/03/09 23:52:43 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/
public interface SessionManagerIfc
{
    /**
     * Return the session associated with the current thread. If one does
     * not exist then create one. If it cannot create a session then throw
     * FailedToCreatedSessionException exception. If one already exists then
     * simply return a reference to it.
     *
     * @return      SessionIfc
     * @exception   FailedToCreateSessionException   
     */
    public SessionIfc getSession()
        throws FailedToCreateSessionException;

    /**
     * Destroy the session associated with thew current thread. If one does not
     * exist then do nothing. If it cannot destroy the session then throw
     * FailedToDestroySessionException exception.
     *
     * @exception   FailedToDestroySessionException
     */
    public void destroySession()
        throws FailedToDestroySessionException;
}
