/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: TransactionStatus.java,v 1.1 2000/03/09 23:52:43 jima Exp $
 *
 * Date         Author  Changes
 * 1/24/2000   jima    Created
 */

package org.exolab.core.foundation;


/**
 * This class defines the various states of a transaction.A transaction can be
 * in one three possible states.
 * <p>
 * OPENED       a begin has been issues and the transaction is in progress
 * COMMITTED    the transaction has been successfully commited
 * ROLLOEDBACK  the transaction has been successfully rolled back
 * <p>
 * A lot of these concepts have been borrowed from Castor
 * 
 * @version     $Revision: 1.1 $ $Date: 2000/03/09 23:52:43 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/

public final class TransactionStatus 
{
	/**
	 * Limit the creation scope of this object. All instances of this 
     * class are created at class initialisation time.
	 *
	 * @param	    String		state name
	 */
	private TransactionStatus(String status) 
	{
		status_ = status;
		ord_ = upperSentinel__++;
	}

	/**
	 * Returns the ordinal value for this state
	 *
	 * @return      int	        
	 */
	public int getOrd() 
	{
		return ord_; 
	}

   /**
	* Returns the name of this state
	*
	* @return       String      
	*/
	public String toString() 
	{
		return status_;
	}

	/**
	 * Returns the number of states in this enumeration
	 *
	 * @return	    int
	 */
	public static int size() 
	{ 
		return upperSentinel__; 
	}

    /**
     * Return true iff the two specified object are of the same type 
     * and their ordinal values are identical.
     *
     * @param       obj         object to compare against
     * @return      boolean     true if objects are equivalent
     */
    public boolean equals(Object obj)
    {
        boolean result = false;

        if (obj instanceof TransactionStatus)
        {
            if (((TransactionStatus)obj).getOrd() == getOrd())
            {
                result = true;
            }
        }

        return result;
    }

    /**
     * Check if the transaction status is set to OPENED
     *
     * @return      boolean         true if opened, false is otherwise
     */
    public boolean isOpened()
    {
        return this.equals(TransactionStatus.OPENED);
    }

	/**
	 * Instatiate instances related to each of the enumerations
	 */
	public static final TransactionStatus OPENED = 
        new TransactionStatus("opened");
	public static final TransactionStatus CLOSED = 
        new TransactionStatus("closed");
	public static final TransactionStatus COMMITTED = 
        new TransactionStatus("committed");
	public static final TransactionStatus ABORTED = 
        new TransactionStatus("aborted");

    private static int upperSentinel__ = 0;

	private String status_;
	private int ord_;
}   