/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: VectorIfc.java,v 1.3 2000/04/30 23:50:18 mourikis Exp $
 *
 * Date         Author  Changes
 * 2/23/2000    jima    Created
 * 3/05/2000    jima    Constrained the interface to apply to 
 *                      PersistentCapableIfc
 */
package org.exolab.core.foundation;


import java.util.Enumeration;
import java.io.Serializable;

/**
 * This interface abstracts a persistent capable Vector data structure so 
 * that it can be mapped by various datastores. It basically uses the JDK1.1.2
 * java.util.Vector to derive the interface and applies only to 
 * PersistentCapableIfc instances.
 * <p>
 * Note that the object is also Serializable
 *
 * @version     $Revision: 1.3 $ $Date: 2000/04/30 23:50:18 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 **/
public interface VectorIfc
    extends PersistentCapableIfc, Serializable
{
    /**
     * Return the size of the vector
     *
     * @return      int
     */
    public int size();

    /**
     * Check to see of the vector is empty.
     *
     * @return      boolean     true if it is empty and false otherwise
     */
    public boolean isEmpty();

    /**
     * Return an enumeration of all elements in the Vector
     *
     * @return      Enumeration
     */ 
    public Enumeration elements();

    /** 
     * Test to see whether the specified object exists
     *
     * @param       element     element to test for
     * @return      boolean     true if it is present and false otherwise
     */
    public boolean contains(PersistentCapableIfc element);

    /**
     * Return the index of the specified element or -1 if it does notr exist
     *
     * @param       element     element to search for
     * @return      int         index of the found element or -1.
     */
    public int indexOf(PersistentCapableIfc element);

    /**
     * Return the object at the specified index or null if one does not
     * exist
     *
     * @param       index                   index to query on
     * @return      PersistentCapableIfc    founnd object or null
     */
    public PersistentCapableIfc elementAt(int index);

    /**
     * Return the first element in the Vector or null if it is empty
     *
     * @return      PersistentCapableIfc    first element in the Vector or null
     */
    public PersistentCapableIfc firstElement();

    /**
     * Return the last element in the Vector or null if it is empty
     *
     * @return      PersistentCapableIfc    last element in the Vector or null
     */
    public PersistentCapableIfc lastElement();

    /**
     * Remove the element at the specified index
     *
     * @param       index       index of element to remove
     */
    public void removeElementAt(int index);

    /**
     * Add the specified element to the end of the vector
     *
     * @param       element     element to add
     */
    public void addElement(PersistentCapableIfc element);

    /**
     * Remove the specified element from the Vector.
     *
     * @param       element     element to remove
     */
    public boolean removeElement(PersistentCapableIfc element);

    /**
     * Remove all the elements from the Vector. Functionally, equivalent to 
     * the clear method
     */
    public void removeAllElements();

    /**
     * Return the objects in the Vector as an array.
     *
     * @return      Object[]
     */
    public Object[] toArray();

    /** 
     * Return the object at the specified index or null
     *
     * @return      PersistentCapableIfc
     */
    public PersistentCapableIfc get(int index);

    /** 
     * Add the specified element to the end of the Vector
     *
     * @param       element     element to add
     * @return      boolean     true if successful and false otherwise
     */
    public boolean add(PersistentCapableIfc element);

	
	/**
	 * Set the object at the location given, overwriting the existing object 
	 * if it exists. An additional constraint applies
     * to element in that it must also be of type PersistentCapableIfc
	 *
     * @param       index       index 
     * @param       element     element to add
	 *
     */
	public void set(int index, PersistentCapableIfc element);
	

    /**
     * Remove the specified element from the Vector
     *
     * @param       element     element to remove
     * @return      boolean     true if successful; and false otherwise 
     */
    public boolean remove(PersistentCapableIfc element);

    /**
     * Add the element at the specified index
     *
     * @param       index       index 
     * @param       element     element to add
     */
    public void add(int index, PersistentCapableIfc element);

    /**
     * Remove the element at the specified index and return it to the 
     * client
     *
     * @param       index                   index to query
     * @return      PersistentCapableIfc    returned object
     */
    public PersistentCapableIfc remove(int index);

    /**
     * Clear the vector
     */
    public void clear();
}

