/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2002 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: ChannelListenerAcceptor.java,v 1.1 2002/02/17 20:48:50 tima Exp $
 */
package org.exolab.core.messenger;


/**
 * A <code>ChannelHandler</code> that accepts a single {@link Channel} and 
 * binds a {@link ChannelListener} to it.
 *
 * @version     $Revision: 1.1 $ $Date: 2002/02/17 20:48:50 $
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 * @see         Channel
 * @see         ChannelListener
 * @see         Connection#accept(String, ChannelListener)
 */
public class ChannelListenerAcceptor implements ChannelHandler {

    /**
     * The owning connection
     */
    private Connection _connection;

    /**
     * The channel name
     */
    private String _name;

    /**
     * The channel listener
     */
    private ChannelListener _listener;

    
    /**
     * Construct a new <code>ChannelListenerAcceptor</code>
     *
     * @param connection the owning connection
     * @param name the channel name
     * @param listener the channel listener
     * @throws IllegalArgumentException if any argument is null
     */
    public ChannelListenerAcceptor(Connection connection, String name,
                                   ChannelListener listener) {
        if (connection == null) {
            throw new IllegalArgumentException(
                "Argument 'connection' is null");
        }
        if (name == null) {
            throw new IllegalArgumentException("Argument 'name' is null");
        }
        if (listener == null) {
            throw new IllegalArgumentException("Argument 'listener' is null");
        }
        _connection = connection;
        _name = name;
        _listener = listener;
    }

    /**
     * Handle a new channel.<p>
     * This implementation binds the listener to the channel, and prevents
     * any more channels from being opened.
     *
     * @param channel the new channel
     */
    public synchronized void opened(Channel channel) {
        channel.setChannelListener(_listener);
        _connection.close(_name);
    }

} //-- ChannelListenerAcceptor
