/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2002 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: Connector.java,v 1.4 2002/02/21 09:49:41 tima Exp $
 */
package org.exolab.core.messenger;

// java
import java.net.MalformedURLException;
import java.rmi.RemoteException;


/**
 * A Connector is a factory for {@link Connection} and 
 * {@link ManagedConnectionAcceptor} objects, for a specific communications 
 * protocol.
 *
 * @version     $Revision: 1.4 $ $Date: 2002/02/21 09:49:41 $
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 * @see         Connection
 * @see         ConnectionAcceptor
 * @see         Messenger
 */
public interface Connector {

    /**
     * Returns a connection for the specified URI and connection properties
     *
     * @param uri the connection address
     * @param properties connection properties (may be null)
     * @return a Connection if the connector finds an acceptable match,
     * otherwise <code>null</code>
     * @throws MalformedURLException if <code>uri</code> is an invalid URI
     * @throws RemoteException if a connection cannot be established
     * @throws IllegalArgumentException if <code>uri</code> is null
     */
    public Connection getConnection(String uri, 
                                    ConnectionProperties properties) 
        throws MalformedURLException, RemoteException;

    /**
     * Returns a connection acceptor for the specified URI and connection 
     * properties
     *
     * @param uri the address to accept connections on
     * @param properties connection properties (may be null)
     * @return a connection acceptor, for accepting connection requests on the
     * specified URI
     * @throws MalformedURLException if <code>uri</code> is an invalid URI
     * @throws RemoteException if an I/O error occurs
     * @throws IllegalArgumentException if <code>uri</code> is null
     */
    public ManagedConnectionAcceptor getManagedConnectionAcceptor(
        String uri, ConnectionProperties properties) 
        throws MalformedURLException, RemoteException;

    /**
     * Returns the protocol that the connector supports
     *
     * @return the protocol that the connector supports
     */
    public String getScheme();

} //-- Connector
