/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2002 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: ResourceAllocator.java,v 1.1 2002/02/15 08:40:05 tima Exp $
 */
package org.exolab.core.messenger;

// exolab
import org.exolab.core.threadPool.ThreadPool;


/**
 * This class allocates resources for {@link Connection} instances
 *
 * @version     $Revision: 1.1 $ $Date: 2002/02/15 08:40:05 $
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 */
public class ResourceAllocator {

    /**
     * The thread pool
     */
    private ThreadPool _threads;

    /**
     * The limits to be placed on connections
     */
    private Limits _limits;

    /**
     * The seed for connection identifiers
     */
    private int _connectionSeed = 0;


    public ResourceAllocator(int threads, Limits limits) {
        if (limits == null) {
            throw new IllegalArgumentException("Argument 'limits' is null");
        }
        _limits = limits;
        _threads = new ThreadPool("Messenger", threads, true);
    }

    public synchronized Context allocateContext() {
        int id = ++_connectionSeed;
        return new Context(id, _limits, _threads);
    }

} //-- ResourceAllocator
