/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2002 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: SystemChannel.java,v 1.2 2002/02/17 20:46:54 tima Exp $
 */
package org.exolab.core.messenger;

// java
import java.rmi.RemoteException;


/**
 * A SystemChannel is reponsible for managing channel open and close
 * requests on a {@link Connection}.<p>
 * Typically, there will be one SystemChannel per Connection. This channel
 * is not accessible to users.
 *
 * @version     $Revision: 1.2 $ $Date: 2002/02/17 20:46:54 $
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 * @see         Channel
 * @see         ChannelAcceptor
 * @see         Connection
 */
public interface SystemChannel extends Channel {

    /**
     * Open a channel for the specified name. A {@link ChannelAcceptor} must
     * be available on the other end of the connection, accepting requests
     * for the named channel.
     *
     * @param name the channel name
     * @return an open channel
     * @throws RemoteException if an error occurs opening the channel
     * @throws InvalidChannelException there is no ChannelAcceptor listening 
     * for requests for the specified name
     * @throws IllegalArgumentException if <code>name</code> is null
     */
    public Channel open(String name) throws RemoteException;

    /**
     * Close a channel
     *
     * @param channel the channel to close
     * @throws RemoteException if an error occurs closing the channel
     * @throws IllegalArgumentException if <code>channel</code> is null
     */
    public void close(Channel channel) throws RemoteException;

} //-- SystemChannel
