/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2002 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: URIHelper.java,v 1.1 2002/02/21 09:46:11 tima Exp $
 */
package org.exolab.core.messenger.tcp;

// java
import java.net.MalformedURLException;

// exolab
import org.exolab.core.util.URI;


/**
 * Helper class for TCP connector URIs
 *
 * @version     $Revision: 1.1 $ $Date: 2002/02/21 09:46:11 $
 * @author      <a href="mailto:tima@intalio.com">Tim Anderson</a>
 * @see         TcpConnector
 */
class URIHelper {

    /**
     * Helper to return the URI portion relevant to a TcpManagedConnection or
     * TcpManagedConnectionAcceptor
     *
     * @param uri the URI
     */
    public static String getAddress(URI uri) {
        return uri.getScheme() + "://" + uri.getHost() + ":" + uri.getPort();
    }

    /**
     * Helper to parse a URI, and perform validation on common attributes
     *
     * @param uri the URI to parse
     * @return the parsed URI
     * @throws MalformedURLException if <code>uri</code> is an invalid URI
     */
    public static URI getURI(String uri) throws MalformedURLException {
        URI parser;
        try {
            parser = new URI(uri);
        } catch (URI.MalformedURIException exception) {
            throw new MalformedURLException(exception.getMessage());
        }
        if (!parser.getScheme().equals("tcp")) {
            throw new MalformedURLException("Invalid tcp uri:" + 
                                            parser.getScheme());
        }
        if (parser.getPort() == -1) {
            throw new MalformedURLException("Port not specified in uri");
        }
        return parser;
    }
    
} //-- URIHelper
