/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2001 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: ConnectionNotifierIfc.java,v 1.2 2001/07/20 07:47:08 mourikis Exp $
 *
 * Date         Author  Changes
 * 02/14/2001   artw    Created
 */


package org.exolab.core.mipc;

import java.io.Serializable;


/**
 * This interface is implemented by clients that wish to be notified
 * by the MultiplexConnectionServer when it establishes new MultiplexConnections.
 * This is typically used by clients to establish channels on the
 * freshly minted MultiplexConnection.  
 *
 * @version     $Revision: 1.2 $
 * @author      <a href="mailto:artw@wfactor.com">Art Whitten</a>
 * @see         org.exolab.core.mipc.MultiplexConnectionServer
 * @see         org.exolab.core.mipc.MultiplexConnection
 **/

public interface ConnectionNotifierIfc
{
	/**
	 * A new connection has been established.  This callback is invoked before
	 * messages are received.  It is a great time to establish channel 
	 * consumers on the MultiplexConnection.
	 *
	 * @param newConnection  The new connection that was established.
	 */
	public void connected(MultiplexConnectionIfc newConnection);

} 
