/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2001-2002 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: MessageInput.java,v 1.2 2002/01/18 12:09:42 tima Exp $
 *
 * Date         Author  Changes
 * 02/14/2001   artw    Created
 * 01/15/2002   tima    Refactored to enhance performance
 */

package org.exolab.core.mipc;

// java io
import java.io.IOException;

/**
 * This interface defines the simple signature required by all MessageInput 
 * streams.  
 *
 * @version     $Revision: 1.2 $
 * @author      <a href="mailto:artw@wfactor.com">Art Whitten</a>
 * @author      <a href="mailto:tima@tima.intalio.com">Tim Anderson</a>
 * @see         MessageOutput
 */
interface MessageInput {

    /**
     * Receive a message from the underlying stream
     *
     * @return the received message
     * @throws ClassNotFoundException if the message cannot be deserialized
     * @throws IOException for any error
     */
    public Packet receive() throws ClassNotFoundException, IOException;

    /**
     * De-allocate any resources, and close the underlying stream
     *
     * @throws IOException for any error
     */
    public void close() throws IOException;

} //-- MessageInput
