/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2001-2002 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: MessageOutputStream.java,v 1.2 2002/01/18 12:09:42 tima Exp $
 *
 * Date         Author  Changes
 * 02/14/2001   artw    Created
 */

package org.exolab.core.mipc;

// java io 
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;


/**
 * This class is a concrete implementation of a MessageOutput.  Its underlying
 * stream is a generic OutputStream.  Access to the raw output stream is 
 * sychronized on the rawstream itself, this means that multiple 
 * MessageOutputStreams can operate on the same raw OutputStream.  
 *
 * @version     $Revision: 1.2 $
 * @author      <a href="mailto:artw@wfactor.com">Art Whitten</a>
 * @see         org.exolab.core.mipc.MessageOutput
 */
class MessageOutputStream implements MessageOutput {

    /**
     * The raw stream in which messages are written to
     */
    private OutputStream _rawOut;

    /**
     * Wraps the rawOut stream
     */
    private DataOutputStream _out;

    /**
     * Construct a new MessageOuputStream that wraps the specified stream.
     *
     * @param out raw output stream to wrap
     */	
    public MessageOutputStream(OutputStream out) {
        _rawOut = out;
        _out = new DataOutputStream(out);
    }

    /**
     * Send a message using the underlying raw stream
     *
     * @param message the packet to send
     * @throws IOException for any error
     */
    public void send(Packet message) throws IOException {
        synchronized (_rawOut) {
            byte[] data = message.getPacket();
            _out.writeInt(data.length);
            _out.write(data);
            _rawOut.flush();
        }
    }

    /**
     * Closes the underlying stream
     *
     * @throws IOException if the stream cannot be closed
     */
    public void close() throws IOException {
        _rawOut.close();
    }

} //-- MessageOutputStream
