/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2001 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: MultiplexConnectionIfc.java,v 1.1 2001/07/20 07:47:08 mourikis Exp $
 *
 * Date         Author      Changes
 * 07/06/2001   mourikis    Created
 */

package org.exolab.core.mipc;


public interface MultiplexConnectionIfc {

    /**
     * Gracefully stop the MultiplexSSLConnection thread.
     */
    public void finish ();

    /** 
     * Set the listener that will be called back when the multiplex connection
     * is dropped
     *
     * @param listener - the disconnection event listener
     */
    public void setDisconnectionEventListener
        (DisconnectionEventListener listener);

    /**
     * The workhorse for the MultiplexConnection.  Typically this method is
     * invoked as a result of calling 'start' on the MultiplexConnection.
     * MultiplexConnection derives from Thread.
     */
    public void run ();

    /**
     * Register a new multiplex channel.  'label' is the identifier for the 
     * channel and 'messageOutput' is where inbound messages are placed.  
     * If there is an existing channel, it is overwritten.
     * 
     * The return value is a MessageOutput that is used by the channel 
     * producer to send messages.  The interface encapsulates a 
     * MutliplexOutputStream.
     *
     * @param label The channel name to register for
     * @param messageOutput Inbound messages are placed here
     * @return MessageOuput Channel producers place outbound message on this 
     *  stream.
     */
    public MessageOutput register(String label, MessageOutput messageOutput);

    /**
     * Deregister a multiplex channel.  
     *
     * @param label The channelname to unregister
     */
    public void deregister( String label );

    /** 
     * This is called by the demultiplexer when the channel has been abnormally
     * terminated. It will call finish on the connection and then notify any
     * registered listener
     */
    void disconnected();

    /**
     * Return the hostname that this connection bound too
     *
     * @return String - the host name
     */
    public String getHost();

    /**
     * Return the port that this connection is bound too
     *
     * @return int - the port number
     */
    public int getPort();
    
} // End MultiplexConnection
