/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2001 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: MultiplexConnectionServerIfc.java,v 1.1 2001/07/20 07:47:08 mourikis Exp $
 *
 * Date         Author      Changes
 * 02/14/2001   mourikis    Created
 */

package org.exolab.core.mipc;

import java.io.IOException;

/**
 * Create a listener SSL socket and wait for requests to come in. When a 
 * MultiplexConnection is made, a client is notified so that it can initialize
 * channels on the freshly minted MultiplexConnection.
 *
 * This class derives from Thread and is meant to be spawned off as a separate
 * Thread.
 *
 * 
 * @version     $Revision: 1.1 $
 * @author      <a href="mailto:mourikis@intalio.com">Jim Mourikis</a>
 * @see         org.exolab.core.mipc.MultiplexConnectionServerIfc
 **/


public interface MultiplexConnectionServerIfc
{
	/**
	 * Start the service running. This call never returns until the service
	 * is shutdown.
	 *
	 * When a connection is made, a new MultiplexConnection is created and 
	 * spawned off as a separate thread.  The new threads are created in the
	 * ThreadGroup connections_ so that we can keep track of the active threads.
	 *
	 */
	public void run();
	

	/**
	 * Shutdown all active connections, including the server connection.
	 *
	 * @exception IOException If the service reports an error during shutdown.
	 */
	public void shutdownAll() throws IOException;

	
	/**
	 * Return the port number this server is listening on.
	 *
	 * @return int The port number the server is using.
	 */
	public int getPort();


	/**
	 * Return the machines address.
	 * 
	 * @return String The machines Ip address.
	 */
	public String getHost();


} // End MultiplexConnectionServerIfc
