/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2001 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: QueueInputStream.java,v 1.2 2002/01/18 12:09:42 tima Exp $
 *
 * Date         Author  Changes
 * 02/14/2001   artw    Created
 */

package org.exolab.core.mipc;

import java.io.*;

/**
 * A MessageInput implementation that uses a Queue as its source
 * for messages.  The messages are likely placed on the Queue by
 * using the QueueOutputStream.
 *
 * The underlying Queue is threadsafe, which means that multiple 
 * QueueInputStreams and QueueOutputStreams can work on the same underlying
 * queue.
 *
 * @version     $Revision: 1.2 $
 * @author      <a href="mailto:artw@wfactor.com">Art Whitten</a>
 * @see         Queue;
 * @see         QueueOutputStream;
 * @see         MessageInput;
 */
class QueueInputStream implements MessageInput {

    /**
     * The source of messages
     */
    private Queue _queue;

    /**
     * Create a new QueueInputStream on the given Queue.
     *
     * @param queue the queue to use for a source of messages.
     */
    public QueueInputStream(Queue queue) {
        _queue = queue;
    }

    /**
     * Receive a message from the underlying stream, in this case the Queue
     *
     * @return the received message
     * @throws IOException for any error
     */
    public Packet receive() throws IOException {
        Packet result = null;
        try {
            result = (Packet) _queue.remove();
            if (result == null) {
                throw new InterruptedIOException("Empty packet was returned");
            }
        } catch (InterruptedException exception) {
            throw new InterruptedIOException("queue_.remove()");
	}
        return result;
    }

    /**
     * Closes the queue
     */
    public void close() {
        _queue.close();
    }

} //-- QueueInputStream
