/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999-2001 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: BasicService.java,v 1.8 2003/06/09 06:28:41 tanderson Exp $
 *
 * Date         Author  Changes
 * 12/23/1999   jima    Created
 * 03/01/2000   jima    Changed the constructor to only take a name. There
 *                      is no need for the Runnable arg, since this object
 *                      is Runnable
 */

package org.exolab.core.service;

// exolab core
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;


/**
 * BasicService is a service implementation that will run the service in a 
 * separate thread. Derived class must define an implementation for the 'run' 
 * method.
 *
 * @version     $Revision: 1.8 $ $Date: 2003/06/09 06:28:41 $
 * @author      <a href="mailto:jima@intalio.com">Jim Alateras</a>
 * @see         Service
 */
public abstract class BasicService extends Service implements Runnable {

    /**
     * The service thread
     */
    private transient Thread _thread = null;

    /**
     * The default constructor for the BasicService does nothing.
     */
    protected BasicService() {
    }

    /**
     * Instantiates a service with the specified name
     * 
     * @param name the unique name of the service       
     */
    protected BasicService(String name) {
        super(name);
    }

    /**
     * Start the service.
     *
     * @throws ServiceException if the service fails to start, or is already
     * running
     */
    public synchronized void start() throws ServiceException {
        super.start();
        _thread = new Thread(this, getName());
        _thread.start();
    }

    /**
     * Stop the service.
     *
     * @throws ServiceException if the service fails to stop, or is already
     * stopped
     */
    public synchronized void stop() throws ServiceException {
        if (_thread != null) {
            setState(ServiceState.STOPPED);
            _thread.interrupt();
            _thread = null;
        } else {
            throw new ServiceException("Failed to stop service " + this); 
        }
    }

    /**
     * Return the state of the object as a string
     *
     * @return a string form of the object state
     */
    public String toString() {
        StringBuffer buf = new StringBuffer("BasicService:[");
        buf.append("name=");
        buf.append(getName());
        buf.append("thread=");
        buf.append(_thread);
        buf.append("state=");
        buf.append(getState());
        buf.append("]");
        return buf.toString();
    }
    
} //-- BasicService
