/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999-2001 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * Date         Author  Changes
 * 12/23/1999   jima    Created
 */

package org.exolab.core.service;

// java utility
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

// exolab core
import org.apache.commons.logging.LogFactory;


/**
 * The ServiceManager is based on the singleton design pattern and is 
 * responsible for managing a collection of {@link Serviceable} objects. 
 * <br>
 * All serviceable objects are named and no two sevices can have the same 
 * name. 
 * <p>
 * The ServiceManager singleton allows the client to manage a group of 
 * Serviceable entities.
 *
 * @version     $Revision: 1.8 $ $Date: 2003/06/09 06:28:41 $
 * @author      <a href="mailto:jima@intalio.com">Jim Alateras</a>
 * @see         BasicService
 * @see         Service
 * @see         ServiceGroup
 * @see         Serviceable
 */
public class ServiceManager extends ServiceGroup {

    /**
     * The singleton instance of the ServiceManager
     */
    private static ServiceManager _instance = null;

    /**
     * Construct a new service manager
     */
    private ServiceManager() {
    }

    /**
     * Return the single instance of this class. If need be, create it before
     * returning it.
     *
     * @return the ServiceManager singleton
     */
    public static synchronized ServiceManager instance() {
        if (_instance == null) {
            _instance = new ServiceManager();
        }
        return _instance;
    }

} //-- ServiceManager
