/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999-2001 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: Serviceable.java,v 1.2 2001/10/07 07:28:05 tima Exp $
 *
 * Date         Author  Changes
 * 12/23/1999   jima    Created
 */

package org.exolab.core.service;

// java io
import java.io.Serializable;


/**
 * This interface defines a protocol for serviceable objects. A serviceable 
 * object is one that can be controlled through start, stop etc. 
 *
 * @version     $Revision: 1.2 $ $Date: 2001/10/07 07:28:05 $
 * @author      <a href="mailto:jima@intalio.com">Jim Alateras</a>
 * @see         BasicService
 */
public interface Serviceable extends Serializable {

    /**
     * Start the service.
     *
     * @throws ServiceException if the service fails to start, or is already
     * running
     */
    public void start() throws ServiceException;
    
    /**
     * Stop the service.
     *
     * @throws ServiceException if the service fails to stop, or is already
     * stopped
     */
    public void stop() throws ServiceException;

    /**
     * Convenience method for restarting the service. This operation can
     * be called regardless the current state of the service.
     *
     * @throws ServiceException if the service fails to restart
     */
    public void restart() throws ServiceException;
    
     /**
      * Return the current state of the service
      *
      * @return the current state of the service
      */
    public ServiceState getState();

    /**
     * Return the name of the service
     *
     * @return the name of the service
     */
    public String getName();

} //-- Serviceable
