/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: BaseException.java,v 1.1 2000/03/03 06:41:30 jima Exp $
 *
 * Date         Author  Changes
 * 3/01/2000    jima    sourced from the Castor project; formely known as 
 *                      org.exolab.castor.util.CastorEzception
 */


package org.exolab.core.util;


import java.io.PrintStream;
import java.io.PrintWriter;
import org.exolab.core.util.Messages;


/**
 * Parent class of all Castor exceptions. The constructor accepts the
 * error code and a number of arguments. It retrieves the exception
 * text for the current locale and formats it using the arguments.
 * The textual error code can be retrieved by calling {@link
 * #getErrorCode}.
 *
 * @author <a href="arkin@exoffice.com">Assaf Arkin</a>
 * @version $Revision: 1.1 $ $Date: 2000/03/03 06:41:30 $
 */
public class BaseException
    extends Exception
{


    private String _errorCode = "error.unknown";


    protected BaseException( String message )
    {
        super( Messages.message( message ) );
        _errorCode = message;
    }


    protected BaseException( String message, Object arg1 )
    {
        super( Messages.format( message, arg1 ) );
        _errorCode = message;
    }


    protected BaseException( String message, Object arg1, Object arg2 )
    {
        super( Messages.format( message, arg1, arg2 ) );
        _errorCode = message;
    }


    public final String getErrorCode()
    {
        return _errorCode;
    }


    public String toString()
    {
        return _errorCode + ": " + getMessage();
    }


}

