/*
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999-2001,2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: RmiRegistryService.java,v 1.8 2003/06/09 06:28:42 tanderson Exp $
 *
 * Date         Author      Changes
 * 12/23/1999   romefort    Created
 * 03/13/200    jima        Placed in different package and changed to
 *                          use generic configuration
 */

package org.exolab.core.util;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;

import org.exolab.core.service.Service;
import org.exolab.core.service.ServiceException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * RmiRegistryService is a service that starts an embedded RMI registry.
 * 
 * @version     $Revision: 1.8 $ $Date: 2003/06/09 06:28:42 $
 * @author      romefort
 * @see         BasicService
 */
public class RmiRegistryService extends Service {

    /**
     * The default service name
     */
    public static final String DEFAULT_NAME = "RmiRegistryService";

    /**
     * The default registry port
     */
    public static final int DEFAULT_PORT = 1099;

    /**
     * The port number that the service was started on
     */
    private int _port = DEFAULT_PORT;

    /**
     * The logger
     */
    private static final Log _log = LogFactory.getLog(RmiRegistryService.class);


    /**
     * Construct a service. If the service cannot be started then
     * throw a ServiceException
     *
     * @throws ServiceException
     */
    public RmiRegistryService() throws ServiceException {
        this(DEFAULT_NAME, DEFAULT_PORT);
    }

    /**
     * Construct the service to run on the specified port. 
     *
     * @param port the port number
     * @throws ServiceException if the registry cannot be created for the port
     */
    public RmiRegistryService(int port) throws ServiceException {
        this(DEFAULT_NAME, port);
    }

    /**
     * Construct an instance of this service with the specified name.
     *
     * @param name the name of the service
     * @param port the port number
     * @throws ServiceException if the registry cannot be created for the port
     */
    public RmiRegistryService(String name, final int port) 
        throws ServiceException {
        super(name);
        try {
            _port = port;
            LocateRegistry.createRegistry(_port);
            _log.info("Instantiated the " + name + " service on port "
                + _port );
        } catch (RemoteException exception) {
            throw new ServiceException("Failed to create RmiRegistryService " +
                                       exception);
        }
    }

    /**
     * Return the port that the registry is listening on
     *
     * @return the registry port
     */
    public int getPort() {
        return _port;
    }

} //-- RmiRegistryService
