/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: OpenSpacesTestHarness.java,v 1.10 2001/10/07 07:28:06 tima Exp $
 *
 * Date         Author  Changes
 * 2/16/2000    jima    Created 
 */

package org.exolab.testharness;
                                             
import java.lang.Throwable;
import java.io.PrintStream;
import java.util.Enumeration;
import java.util.Vector;
import org.exolab.core.CWCommandLine;
import org.exolab.jtf.CWTestHarness;
import org.exolab.jtf.CWTestHarness;
import org.exolab.jtf.CWBaseApplication;
import org.exolab.exceptions.CWClassConstructorException;


/**
 * This is the mainline for the OpenSpaces test harness. This test harness can 
 * execute one or more tests depending on the input parameter. It also follows
 * the signelton design pattern.
 */
public class OpenSpacesTestHarness extends CWBaseApplication
{
    // define the collection of test categories to associate with 
    // this test hanress
    static Vector categories__ = new Vector();
    static
    {
        categories__.addElement(
            "org.exolab.testharness.core.recman.RecordManagerCT"); 
        categories__.addElement(
            "org.exolab.testharness.core.recman.SessionManagerCT");    
        categories__.addElement(
            "org.exolab.testharness.core.recman.SessionCT");    
		categories__.addElement(
            "org.exolab.testharness.core.threadPool.ThreadPoolCT"); 
		categories__.addElement(
            "org.exolab.testharness.core.foundation.LruObjectCacheCT");
    }

	static public void main(String args[])
	{
        try
        {
            OpenSpacesTestHarness harness = new OpenSpacesTestHarness("openspaces");
            harness.run(args);
        }
        catch (Exception exception)
        {
            exception.printStackTrace();
        }
	}

    /**
     * Instnatiate an instance of this testharness with the specified name. Call the base
     * class. The CWClassConstructorException is raised if it cannot construct the object
     *
     * @param       name            name of the test harness
     * @exception   CWClassConstructorException
     */
    public OpenSpacesTestHarness(String name) 
        throws CWClassConstructorException
	{
        super(name);
	}

    // define for CWBaseApplication.getCategoryNames
    protected Enumeration getCategoryClassNames()
    {
        return categories__.elements();
    }


    // define for CWBaseApplication.getApplicationName
    protected String getApplicationName()
    {
        return "org.exolab.testharness.OpenSpacesTestHarness";
    }
}			
