/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: SessionCT.java,v 1.8 2001/09/04 02:30:07 jima Exp $
 *
 * Date         Author      Changes
 * 03/08/1999   jima        Created
 */

package org.exolab.testharness.core.recman;

import java.util.Properties;
import java.util.Enumeration;
import java.util.Hashtable;
import java.io.File;
import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import org.exolab.jtf.CWTestCategory;
import org.exolab.jtf.CWTestCase;
import org.exolab.jtf.CWVerboseStream;
import org.exolab.exceptions.CWClassConstructorException;
import org.exolab.core.database.recman.PMDSessionManager;
import org.exolab.core.database.recman.PMDSession;
import org.exolab.core.database.recman.PageManagedDatabase;
import org.exolab.core.database.recman.PMDVector;
import org.exolab.core.database.recman.PMDHashMap;
import org.exolab.core.database.recman.PMDTreeMap;
import org.exolab.core.database.recman.PMDTreeSet;
import org.exolab.core.database.recman.PMDHandle;
import org.exolab.core.foundation.FailedToDestroySessionException;
import org.exolab.core.foundation.CollectionManagerIfc;
import org.exolab.core.foundation.HandleIfc;
import org.exolab.core.foundation.SessionIfc;
import org.exolab.core.foundation.VectorIfc;
import org.exolab.core.foundation.HashMapIfc;
import org.exolab.core.foundation.TreeMapIfc;
import org.exolab.core.foundation.TreeSetIfc;
import org.exolab.core.foundation.PersistentObject;


/**
 * This file includes test cases for the PMDSessionManager which provides
 * session management capabilties to the PageManagedDatabase. The PMDSession-
 * Manager is responsible for co-ordinating access to the database through 
 * multiple threads. 
 *
 * @version     $Revision: 1.8 $ $Date: 2001/09/04 02:30:07 $
 * @author      <a href="mailto:jima@exoffice.com">Jim Alateras</a>
 * @see         org.exolab.jtf.CWTestCateogry
 * @see         org.exolab.jtf.CWTestCase
 */
public class SessionCT
    extends CWTestCategory 
{
    /**
     * The constructor is responsible for constructing a test category and
     * adding the suite of test cases. It throws CWClassConstructorException
     * if it cannot construct the category
     *
     * @exception   CWClassConstructorException   
     */
    public SessionCT() 
        throws CWClassConstructorException
	{
		super("session", "PMD Session Component");
		CWTestCase tc;

		tc = new SessionComponentTC01();
		add(tc.name(), tc, true);
		tc = new SessionComponentTC02();
		add(tc.name(), tc, true);
		tc = new SessionComponentTC03();
		add(tc.name(), tc, true);
		tc = new SessionComponentTC04();
		add(tc.name(), tc, true);
		tc = new SessionComponentTC05();
		add(tc.name(), tc, true);
		tc = new SessionComponentTC06();
		add(tc.name(), tc, true);
		tc = new SessionComponentTC07();
		add(tc.name(), tc, true);
		tc = new SessionComponentTC08();
		add(tc.name(), tc, true);
    }
}

/**
 * This test case executes the following
 * 1. Create an new database
 * 1. Attach it to the session manager
 * 3. get a session for this thread multiple times
 * 4. destroy the session multiple times
 * 5. Clean up by rmeoving the original database.
 */
class SessionComponentTC01 extends CWTestCase
{
    static private final String dbname__ = "pgman";

    /**
     * Calls the super class and throws CWClassConstructorException if it fails
     *
     * @exception   CWClassConstructorException
     */
    public SessionComponentTC01() 
        throws CWClassConstructorException
	{
		super("TC01", "Test the collection creration bind, unbind and lookup");
	}

	/**
	 * Remove the database from the current directory.
	 */
	public void preExecute()
	{
        (new File(dbname__ + ".db")).delete();
        (new File(dbname__ + ".lg")).delete();
        (new File(dbname__ + ".lock")).delete();
	}
	
    /**
     * Execute the test case described in the class header. The CWVerboseStream
     * is used to display detailed information on the test case execution
     *
     * @param           stream          this is where verbose info is streamed
     * @return          boolean         true if successful; false otherwise
     */
	public boolean run(CWVerboseStream stream)
	{
		boolean result = true;

		try
		{
            PageManagedDatabase db = new PageManagedDatabase(dbname__);
            PMDSessionManager.init(db);

            SessionIfc session = PMDSessionManager.instance().getSession();
            if (session == null)
            {
                stream.writeVerbose("Couldn't create session for thread");
            }
            
            // create a vector
            VectorIfc vector = session.getCollectionManager().createVector();
            if (vector == null)
            {
                stream.writeVerbose("Failed to create a vector");
                result = false;
            }

            // create a hashmap
            HashMapIfc map = session.getCollectionManager().createHashMap();
            if (map == null)
            {
                stream.writeVerbose("Failed to create a map");
                result = false;
            }

            // bind the collections
            session.getCurrentTransaction().begin();
            session.bind("vector", vector);
            session.bind("map", map);

            // lookup the collections
            if (session.lookup("vector") == null)
            {
                stream.writeVerbose("Failed to bind vector");
                result = false;
            }

            if (session.lookup("map") == null)
            {
                stream.writeVerbose("Failed to bind map");
                result = false;
            }

            // unbind the collections
            session.getCurrentTransaction().commit();
            session.getCurrentTransaction().begin();
            session.unbind("vector");
            session.unbind("map");

            if (session.lookup("vector") != null)
            {
                stream.writeVerbose("Failed to unbind vector");
                result = false;
            }

            if (session.lookup("map") != null)
            {
                stream.writeVerbose("Failed to unbind map");
                result = false;
            }
            session.getCurrentTransaction().commit();
            PMDSessionManager.instance().destroySession();
            db.close();
 		}
		catch(Exception exception)
		{
			System.out.println("[ERR : SessionComponentTC01] " + 
                exception.getMessage());
		}

		return result;
	}

	/**
	 * Delete the file name after the method has completed.
	 */
	public void postExecute()
    {
        (new File(dbname__ + ".db")).delete();
        (new File(dbname__ + ".lg")).delete();
        (new File(dbname__ + ".lock")).delete();
	}
}


/**
 * This test case executes the following
 * 1. Create an new database and attach a session manager to it
 * 2. create a session for the main thread
 * 3. fire of 20 threads that both access the root
 * 4. let the main thread wait for the other 2 threads to finish
 * 5. check that there are 2 root objects and access them
 */
class SessionComponentTC02 extends CWTestCase
{
    static private final String dbname__ = "pgman";
    static private int NUM_OF_THREADS = 30;

    /**
     * Calls the super class and throws CWClassConstructorException if it fails
     *
     * @exception   CWClassConstructorException
     */
    public SessionComponentTC02() 
        throws CWClassConstructorException
	{
		super("TC02", "Test " + NUM_OF_THREADS + 
            " threads accessing the root object");
	}

	/**
	 * Remove the database from the current directory.
	 */
	public void preExecute()
	{
        (new File(dbname__ + ".db")).delete();
        (new File(dbname__ + ".lg")).delete();
        (new File(dbname__ + ".lock")).delete();
	}
	
    /**
     * Execute the test case described in the class header. The CWVerboseStream
     * is used to display detailed information on the test case execution
     *
     * @param           stream          this is where verbose info is streamed
     * @return          boolean         true if successful; false otherwise
     */
	public boolean run(CWVerboseStream stream)
	{
		boolean result = false;

		try
		{
            PageManagedDatabase db = new PageManagedDatabase(dbname__);
            PMDSessionManager.init(db);

            SessionIfc session = PMDSessionManager.instance().getSession();
            if (session == null)
            {
                stream.writeVerbose("Couldn't create session for thread");
            }
            
            // create two threads and two instances of SimpleClient and then
            // start them
            Thread threads[] = new Thread[NUM_OF_THREADS];
            for (int index = 0; index < NUM_OF_THREADS; index++)
            {
                threads[index] = new Thread(new SessionClient(stream));
                threads[index].start();
            }

            // wait for all threads to complete
            for (int index = 0; index < NUM_OF_THREADS; index++)
            {
                threads[index].join();
            }

            // check that the two sessions are not identical
            stream.writeVerbose("All client threads have finished");

            // now check that there are two roots'
            Enumeration roots = session.getRootNames();

            int index = 0;
            while (roots.hasMoreElements())
            {
                String name = (String)roots.nextElement();
                stream.writeVerbose("Retrieved the root " + name);
                index++;
            }

            if (index == NUM_OF_THREADS)
            {
                result = true;
            }
            else
            {
                System.err.println("Read only " + index + " roots");
            }

            PMDSessionManager.instance().destroySession();
            db.close();
 		}
		catch(Exception exception)
		{
			System.out.println("[ERR : SessionComponentTC02] " + 
                exception.getMessage());
		}

		return result;
	}

	/**
	 * Delete the file name after the method has completed.
	 */
	public void postExecute()
    {
        (new File(dbname__ + ".db")).delete();
        (new File(dbname__ + ".lg")).delete();
        (new File(dbname__ + ".lock")).delete();
	}
}

/**
 * This test case executes the following
 * 1. Create an new database and attach a session manager to it
 * 2. create a session for the main thread
 * 3. fire of 2 threads 
 * 4. first thread binds 1 root object and commits
 * 5  second thread binds 3 root objects and aborts
 * 6. check that there are 1 root objects after all work is done
 */
class SessionComponentTC03 extends CWTestCase
{
    static private final String dbname__ = "pgman";

    /**
     * Calls the super class and throws CWClassConstructorException if it fails
     *
     * @exception   CWClassConstructorException
     */
    public SessionComponentTC03() 
        throws CWClassConstructorException
	{
		super("TC03", "Test concurent commit and abort on same object");
	}

	/**
	 * Remove the database from the current directory.
	 */
	public void preExecute()
	{
        (new File(dbname__ + ".db")).delete();
        (new File(dbname__ + ".lg")).delete();
        (new File(dbname__ + ".lock")).delete();
	}
	
    /**
     * Execute the test case described in the class header. The CWVerboseStream
     * is used to display detailed information on the test case execution
     *
     * @param           stream          this is where verbose info is streamed
     * @return          boolean         true if successful; false otherwise
     */
	public boolean run(CWVerboseStream stream)
	{
		boolean result = false;

		try
		{
            PageManagedDatabase db = new PageManagedDatabase(dbname__);
            PMDSessionManager.init(db);

            SessionIfc session = PMDSessionManager.instance().getSession();
            if (session == null)
            {
                stream.writeVerbose("Couldn't create session for thread");
            }
            
            // create two threads and two instances of SimpleClient and then
            // start them
            Thread threads[] = new Thread[2];

            threads[0] = new Thread(new SessionClient(stream));
            threads[1] = new Thread(new SessionAbortClient(stream));

            threads[0].start();
            threads[1].start();

            // wait for all threads to complete
            for (int index = 0; index < 2; index++)
            {
                threads[index].join();
            }

            // check that the two sessions are not identical
            stream.writeVerbose("All client threads have finished");

            // now check that there are two roots'
            session.getCurrentTransaction().begin();
            stream.writeVerbose("Attempting to get root names");
            Enumeration roots = session.getRootNames();
            stream.writeVerbose("Got root names");
            session.getCurrentTransaction().commit();
            stream.writeVerbose("Committed transaction");

            int index = 0;
            while (roots.hasMoreElements())
            {
                String name = (String)roots.nextElement();
                stream.writeVerbose("Retrieved the root " + name);
                index++;
            }

            if (index == 1)
            {
                result = true;
            }
            else
            {
                System.err.println("Read only " + index + " roots");
            }

            PMDSessionManager.instance().destroySession();
            db.close();
 		}
		catch(Exception exception)
		{
			System.out.println("[ERR : SessionComponentTC03] " + 
                exception.getMessage());
		}

		return result;
	}

	/**
	 * Delete the file name after the method has completed.
	 */
	public void postExecute()
    {
        (new File(dbname__ + ".db")).delete();
        (new File(dbname__ + ".lg")).delete();
        (new File(dbname__ + ".lock")).delete();
	}
}

/**
 * This test case executes the following
 * 1. Create an new database and attach a session manager to it
 * 2. create a session for the main thread
 * 3. create a object in one transaction and commit it
 * 4. update the same object in another transaction and commit it
 * 5. delete the same object in another transaction and abort it
 * 6. delete the same object in another transaction and commit it
 */
class SessionComponentTC04 extends CWTestCase
{
    static private final String dbname__ = "pgman";

    /**
     * Calls the super class and throws CWClassConstructorException if it fails
     *
     * @exception   CWClassConstructorException
     */
    public SessionComponentTC04() 
        throws CWClassConstructorException
	{
		super("TC04", "Test create, update and delete object methods");
	}

	/**
	 * Remove the database from the current directory.
	 */
	public void preExecute()
	{
        (new File(dbname__ + ".db")).delete();
        (new File(dbname__ + ".lg")).delete();
        (new File(dbname__ + ".lock")).delete();
	}
	
    /**
     * Execute the test case described in the class header. The CWVerboseStream
     * is used to display detailed information on the test case execution
     *
     * @param           stream          this is where verbose info is streamed
     * @return          boolean         true if successful; false otherwise
     */
	public boolean run(CWVerboseStream stream)
	{
		boolean result = true;

		try
		{
            PageManagedDatabase db = new PageManagedDatabase(dbname__);
            PMDSessionManager.init(db);

            SessionIfc session = PMDSessionManager.instance().getSession();
            if (session == null)
            {
                stream.writeVerbose("Couldn't create session for thread");
            }
            
            // create the object
            session.getCurrentTransaction().begin();
            PMDVector vector = (PMDVector)session.getCollectionManager().createVector();
            vector.setMinimumObjectSize(20000);
            session.createObject(vector);
            session.bind("multivector", vector);
            session.getCurrentTransaction().commit();

            // in this transaction block retrieve the object, add and element to it 
            // and update it. we also acquired a lock. Do this twnety times
            for (int count = 0; count < 101; count++)
            {
                session.getCurrentTransaction().begin();
                vector = (PMDVector)session.lookup("multivector");
                session.acquireLock(vector, 5000);

                // create a persistent object and then store the handle in the 
                // collection
                PMDInteger item = new PMDInteger(count);
                session.createObject(item);
                vector.addElement(session.createHandle(item));
                
                session.updateObject(vector);
                session.getCurrentTransaction().commit();
            }

            // check that the update has suceeded
            session.getCurrentTransaction().begin();
            vector = (PMDVector)session.lookup("multivector");
            if (vector.size() != 101)
            {
                result = false;
                stream.writeVerbose("There should be 1 element in the vector " +
                    "but on query there is " + vector.size());
            }
            session.getCurrentTransaction().abort();

            // iterate through, resolve the handles and print out the 
            // contents of element in the vector
            session.getCurrentTransaction().begin();
            vector = (PMDVector)session.lookup("multivector");
            session.acquireLock(vector, 5000);

            Enumeration elements = vector.elements();
            while (elements.hasMoreElements())
            {
                PMDHandle handle = (PMDHandle)elements.nextElement();
                stream.writeVerbose("Object value is " + handle.resolve());
            }
            session.getCurrentTransaction().abort();

            // delete the object in this transaction and abort
            session.getCurrentTransaction().begin();
            vector = (PMDVector)session.lookup("multivector");
            session.acquireLock(vector, 5000);
            session.deleteObject(vector);
            session.getCurrentTransaction().abort();

            // check that the delete was not successful
            session.getCurrentTransaction().begin();
            vector = (PMDVector)session.lookup("multivector");
            if (vector == null)
            {
                result = false;
                stream.writeVerbose("The object multivector is not there");
            }
            session.getCurrentTransaction().abort();


            // delete the object in this transaction and commit
            session.getCurrentTransaction().begin();
            vector = (PMDVector)session.lookup("multivector");
            session.acquireLock(vector, 5000);
            session.unbind("multivector");
            session.deleteObject(vector);
            session.getCurrentTransaction().commit();

            // check that the delete was not successful
            session.getCurrentTransaction().begin();
            vector = (PMDVector)session.lookup("multivector");
            if (vector != null)
            {
                result = false;
                stream.writeVerbose("The object multivector is still there");
            }
            session.getCurrentTransaction().abort();

            PMDSessionManager.instance().destroySession();
            db.close();
 		}
		catch(Exception exception)
		{
            result = false;
			System.out.println("[ERR : SessionComponentTC04] " + 
                exception.getMessage());
            exception.printStackTrace();
		}

		return result;
	}

	/**
	 * Delete the file name after the method has completed.
	 */
	public void postExecute()
    {
        (new File(dbname__ + ".db")).delete();
        (new File(dbname__ + ".lg")).delete();
        (new File(dbname__ + ".lock")).delete();
	}
}


/**
 * This test case works in conjunction with TC06 and tests the database
 * access over a number of sessions.
 *
 * 1. Create a session and database
 * 2. Create the root object Top and populate it
 * 3. Change the root object and close the database
 */
class SessionComponentTC05 extends CWTestCase
{
    static private final String dbname__ = "pgman";

    /**
     * Calls the super class and throws CWClassConstructorException if it fails
     *
     * @exception   CWClassConstructorException
     */
    public SessionComponentTC05() 
        throws CWClassConstructorException
	{
		super("TC05", "Part 1 of 2 create a root object and populate it");
	}

	/**
	 * Delete the file name after the method has completed.
	 */
	public void preExecute()
    {
        (new File(dbname__ + ".db")).delete();
        (new File(dbname__ + ".lg")).delete();
        (new File(dbname__ + ".lock")).delete();
	}

    /**
     * Execute the test case described in the class header. The CWVerboseStream
     * is used to display detailed information on the test case execution
     *
     * @param           stream          this is where verbose info is streamed
     * @return          boolean         true if successful; false otherwise
     */
	public boolean run(CWVerboseStream stream)
	{
		boolean result = true;

		try
		{
            PageManagedDatabase db = new PageManagedDatabase(dbname__);
            PMDSessionManager.init(db);

            SessionIfc session = PMDSessionManager.instance().getSession();
            if (session == null)
            {
                stream.writeVerbose("Couldn't create session for thread");
            }
            
            // create the object
            session.getCurrentTransaction().begin();
            PMDHashMap map = 
				(PMDHashMap)session.getCollectionManager().createHashMap();
            session.createObject(map);
            session.bind("Top", map);
            session.getCurrentTransaction().commit();

            session.getCurrentTransaction().begin();
            map = (PMDHashMap)session.lookup("Top");
            session.acquireLock(map, 5000);
            map.put("First", new PMDInteger(1));
            session.updateObject(map);
            session.getCurrentTransaction().commit();

            // check that the update has suceeded
            session.getCurrentTransaction().begin();
            map = (PMDHashMap)session.lookup("Top");
			System.out.println("Value First = " + 
							   ((PMDInteger)map.get("First")).get());
            session.getCurrentTransaction().commit();

            PMDSessionManager.instance().destroySession();
            db.close();
 		}
		catch(Exception exception)
		{
            result = false;
			System.out.println("[ERR : SessionComponentTC05] " + 
                exception.getMessage());
            exception.printStackTrace();
		}

		return result;
	}

	
}


/**
 * This test case works in conjunction with TC05 and tests the database
 * access over a number of sessions.
 *
 * 1. Create a session 
 * 2. Lookup the object Top
 * 3. Change the object and update it.
 */
class SessionComponentTC06 extends CWTestCase
{
    static private final String dbname__ = "pgman";

    /**
     * Calls the super class and throws CWClassConstructorException if it fails
     *
     * @exception   CWClassConstructorException
     */
    public SessionComponentTC06() 
        throws CWClassConstructorException
	{
		super("TC06", "Part 2 of 2 Lookup the root object and change it");
	}

    /**
     * Execute the test case described in the class header. The CWVerboseStream
     * is used to display detailed information on the test case execution
     *
     * @param           stream          this is where verbose info is streamed
     * @return          boolean         true if successful; false otherwise
     */
	public boolean run(CWVerboseStream stream)
	{
		boolean result = true;

		try
		{
            PageManagedDatabase db = new PageManagedDatabase(dbname__);
            PMDSessionManager.init(db);

            SessionIfc session = PMDSessionManager.instance().getSession();
            if (session == null)
            {
                stream.writeVerbose("Couldn't create session for thread");
            }
            
            // Retrive our object
            session.getCurrentTransaction().begin();
            PMDHashMap map = (PMDHashMap)session.lookup("Top");
			if (map != null)
			{
				System.out.println("Value First = " + 
								   ((PMDInteger)map.get("First")).get());
				System.out.println("About to add values");
				map.put("Second", new PMDInteger(500));
				session.updateObject(map);
			}
			else
			{
				System.err.println("Failed to find Top Object");
				
			}
			
            session.getCurrentTransaction().commit();

            PMDSessionManager.instance().destroySession();
            db.close();
 		}
		catch(Exception exception)
		{
            result = false;
			System.out.println("[ERR : SessionComponentTC06] " + 
                exception.getMessage());
            exception.printStackTrace();
		}

		return result;
	}

	
}

/**
 * This test case exercies the CollectionManager through the session interface.
 *
 * 1. Create a session 
 * 2. Get access to the collection manager 
 * 3. Create a HashMap, TreeMap, Vector, TreeSet, TreeMap
 * 4. Store them in the database
 * 5. look them up and read them.
 */
class SessionComponentTC07 extends CWTestCase
{
    static private final String dbname__ = "pgman";

    /**
     * Calls the super class and throws CWClassConstructorException if it fails
     *
     * @exception   CWClassConstructorException
     */
    public SessionComponentTC07() 
        throws CWClassConstructorException
	{
		super("TC07", "Exercise the CollectionManager");
	}

	/**
	 * Delete the file name after the method has completed.
	 */
	public void preExecute()
    {
        (new File(dbname__ + ".db")).delete();
        (new File(dbname__ + ".lg")).delete();
        (new File(dbname__ + ".lock")).delete();
	}

    /**
     * Execute the test case described in the class header. The CWVerboseStream
     * is used to display detailed information on the test case execution
     *
     * @param           stream          this is where verbose info is streamed
     * @return          boolean         true if successful; false otherwise
     */
	public boolean run(CWVerboseStream stream)
	{
		boolean result = true;

		try
		{
            PageManagedDatabase db = new PageManagedDatabase(dbname__);
            PMDSessionManager.init(db);

            SessionIfc session = PMDSessionManager.instance().getSession();
            CollectionManagerIfc collmgr= session.getCollectionManager();
            if ((session == null) ||
                (collmgr == null))
            {
                stream.writeVerbose("Couldn't create session or activate " +
                                    "collection managerfor thread");
            }
            
            // create the different collection types and bind them to the
            // database
            session.getCurrentTransaction().begin();

            VectorIfc vector = collmgr.createVector();
            vector.addElement(new PMDInteger(1));
            session.bind("vector", vector);

            HashMapIfc map = collmgr.createHashMap();
            map.put("first", new PMDInteger(2));
            session.bind("map", map);

            TreeMapIfc treemap = collmgr.createTreeMap();
            treemap.put("first", new PMDInteger(3));
            session.bind("treemap", treemap);

            TreeSetIfc treeset = collmgr.createTreeSet();
            treeset.add(new PMDInteger(4));
            session.bind("treeset", treeset);
            
            session.getCurrentTransaction().commit();

            session.getCurrentTransaction().begin();
            vector = (VectorIfc)session.lookup("vector");
            if (vector == null)
            {
                System.err.println("Failed to retrieve the vector");
                result = false;
            }
            else
            {
                PMDInteger value = (PMDInteger)vector.firstElement();
                if (value.get() != 1)
                {
                    System.err.println("The value of the element is " + 
                        value.get());
                    result = false;
                }
            }

            map = (HashMapIfc)session.lookup("map");
            if (map == null)
            {
                System.err.println("Failed to retrieve the map");
                result = false;
            }
            else
            {
                PMDInteger value = (PMDInteger)map.get("first");
                if (value.get() != 2)
                {
                    System.err.println("The value of the element is " + 
                        value.get());
                    result = false;
                }
            }

            treemap = (TreeMapIfc)session.lookup("treemap");
            if (treemap == null)
            {
                System.err.println("Failed to retrieve the treemap");
                result = false;
            }
            else
            {
                PMDInteger value = (PMDInteger)treemap.get("first");
                if (value.get() != 3)
                {
                    System.err.println("The value of the element is " + 
                        value.get());
                    result = false;
                }
            }

            treeset = (TreeSetIfc)session.lookup("treeset");
            if (treeset == null)
            {
                System.err.println("Failed to retrieve the treeset");
                result = false;
            }
            else
            {
                PMDInteger value = (PMDInteger)treeset.first();
                if (value.get() != 4)
                {
                    System.err.println("The value of the element is " + 
                        value.get());
                    result = false;
                }
            }
            session.getCurrentTransaction().commit();
            PMDSessionManager.instance().destroySession();
            db.close();
 		}
		catch(Exception exception)
		{
            result = false;
			System.out.println("[ERR : SessionComponentTC07] " + 
                exception.getMessage());
            exception.printStackTrace();
		}

		return result;
	}

	/**
	 * Delete the file name after the method has completed.
	 */
	public void postExecute()
    {
        (new File(dbname__ + ".db")).delete();
        (new File(dbname__ + ".lg")).delete();
        (new File(dbname__ + ".lock")).delete();
	}
}


/**
 * This tests the database manager without an object cache and 
 * the database manager with an object cache.
 *
 * 1. Create a database without a cache
 * 2. CReate a root object i.e collection
 * 3. Add 100 entries to the collection
 * 4  Iterate through the collection
 * 5  Get the timing for 3-4
 * 6  Repeat 2-5 with a cache size of 512
 */
class SessionComponentTC08 extends CWTestCase
{
    static private final String dbname__ = "pgman";

    /**
     * Calls the super class and throws CWClassConstructorException if it fails
     *
     * @exception   CWClassConstructorException
     */
    public SessionComponentTC08() 
        throws CWClassConstructorException
	{
		super("TC08", "Performance of the object cache");
	}

	/**
	 * Delete the file name after the method has completed.
	 */
	public void preExecute()
    {
        (new File(dbname__ + ".db")).delete();
        (new File(dbname__ + ".lg")).delete();
        (new File(dbname__ + ".lock")).delete();
	}

    /**
     * Execute the test case described in the class header. The CWVerboseStream
     * is used to display detailed information on the test case execution
     *
     * @param           stream          this is where verbose info is streamed
     * @return          boolean         true if successful; false otherwise
     */
	public boolean run(CWVerboseStream stream)
	{
		boolean result = true;

		try
		{
            PageManagedDatabase db = new PageManagedDatabase(dbname__);
            PMDSessionManager.init(db);

            SessionIfc session = PMDSessionManager.instance().getSession();
            CollectionManagerIfc collmgr= session.getCollectionManager();
            if ((session == null) ||
                (collmgr == null))
            {
                stream.writeVerbose("Couldn't create session or activate " +
                                    "collection managerfor thread");
            }
            
            // create the different collection types and bind them to the
            // database
            long start_time = System.currentTimeMillis();
            session.getCurrentTransaction().begin();
            VectorIfc vector = collmgr.createVector();
            vector.setMinimumObjectSize(10000);
            session.bind("vector", vector);
            session.getCurrentTransaction().commit();

            // add them all to the database
            for (int index = 1; index <= 150; index++)
            {
                session.getCurrentTransaction().begin();
                vector = (VectorIfc)session.lookup("vector");
                PMDInteger integer = new PMDInteger(index);
                session.createObject(integer);
                HandleIfc handle = session.createHandle(integer);
                vector.addElement(handle);
                session.updateObject(vector);
                session.getCurrentTransaction().commit();
            }

            // iterate through them all and print them out
            for (int index = 0; index < 150; index++)
            {
                session.getCurrentTransaction().begin();
                vector = (VectorIfc)session.lookup("vector");
                HandleIfc handle = (HandleIfc)vector.elementAt(index);
                PMDInteger integer = (PMDInteger)handle.resolve();
                if (integer.get() != index + 1)
                {
                    stream.writeVerbose("Comparing " + (index + 1) + " with "
                        + integer.get());
                    result = false;
                }
                session.getCurrentTransaction().commit();
            }
            long duration = System.currentTimeMillis() - start_time;
            System.out.println("It has taken " + duration + 
                "ms to complete the test without a cache");

            // do the same but with a cache defined
            db.setObjectCacheSize(512);
            start_time = System.currentTimeMillis();
            session.getCurrentTransaction().begin();
            vector = collmgr.createVector();
            vector.setMinimumObjectSize(10000);
            session.bind("c_vector", vector);
            session.getCurrentTransaction().commit();

            // add them all to the database
            for (int index = 1; index <= 150; index++)
            {
                session.getCurrentTransaction().begin();
                vector = (VectorIfc)session.lookup("c_vector");
                PMDInteger integer = new PMDInteger(index);
                session.createObject(integer);
                HandleIfc handle = session.createHandle(integer);
                vector.addElement(handle);
                session.updateObject(vector);
                session.getCurrentTransaction().commit();
            }

            // iterate through them all and print them out
            for (int index = 0; index < 150; index++)
            {
                session.getCurrentTransaction().begin();
                vector = (VectorIfc)session.lookup("c_vector");
                HandleIfc handle = (HandleIfc)vector.elementAt(index);
                PMDInteger integer = (PMDInteger)handle.resolve();
                if (integer.get() != index + 1)
                {
                    stream.writeVerbose("Comparing " +  (index + 1) + " with "
                        + integer.get());
                    result = false;
                }
                session.getCurrentTransaction().commit();
            }
            
            long cache_duration = System.currentTimeMillis() - start_time;
            System.out.println("It has taken " + cache_duration + 
                "ms to complete the test with a cache");

            System.out.println("Cache improves performance by " +
                (((double)(duration - cache_duration) / (double)cache_duration) * 100.0) + "%");
            
            
            PMDSessionManager.instance().destroySession();
            db.close();
 		}
		catch(Exception exception)
		{
            result = false;
			System.out.println("[ERR : SessionComponentTC08] " + 
                exception.getMessage());
            exception.printStackTrace();
		}

		return result;
	}

	/**
	 * Delete the file name after the method has completed.
	 */
	public void postExecute()
    {
        (new File(dbname__ + ".db")).delete();
        (new File(dbname__ + ".lg")).delete();
        (new File(dbname__ + ".lock")).delete();
	}
}


/** 
 * This class, when run will connect to the session manager, and 
 * get a session
 */
class SessionClient implements Runnable
{
    /**
     * Instantiate an instance of this class with the specified stream
     *
     * @param       CWVerboseStream
     */
    public SessionClient(CWVerboseStream stream)
    {
        stream_ = stream;
        instance_ = count__++;
    }

    /**
     * Get access to the session manager and create a session for this
     * thread
     */
    public void run()
    {
        for (;;)
        {
            try
            {
                // acquire a session
                session_ = PMDSessionManager.instance().getSession();
                stream_.writeVerbose("SessionClient Created the session " + session_.toString());
    
                // bind a version of the vector
                session_.getCurrentTransaction().begin();
                VectorIfc vector = session_.getCollectionManager().createVector();
                session_.bind(("multivector" + Integer.toString(instance_)), vector);
                session_.getCurrentTransaction().commit();
                stream_.writeVerbose("Destroyed the session " + session_.toString());
                break;
            }
            catch (Exception exception)
            {
                try
                {
                    session_.getCurrentTransaction().abort();
                    stream_.writeVerbose("SessionClient exception " + exception);
                    Thread.sleep((long)(500 + (Math.random() * 300)));
                }
                catch (Exception other)
                {
                    // ignore it
                }
            }
            finally
            {
                try
                {
                    PMDSessionManager.instance().destroySession();
                }
                catch (Exception exception)
                {
                    System.err.println("Failure in SessionClient.run " +
                        exception.toString());
                }
            }
        }
    }

    /**
     * Return the session
     *
     * @return      SessionIfc
     */
    public SessionIfc getSession()
    {
        return session_;
    }


    private SessionIfc session_ = null;
    private CWVerboseStream stream_ = null;
    private int instance_ = 0;

    private static int count__ = 100;
}


/** 
 * This class, when run will connect to the session manager, and 
 * get a session. It binds to several entities and then aborts
 */
class SessionAbortClient implements Runnable
{
    /**
     * Instantiate an instance of this class with the specified stream
     *
     * @param       CWVerboseStream
     */
    public SessionAbortClient(CWVerboseStream stream)
    {
        stream_ = stream;
        instance_ = count__++;
    }

    /**
     * Get access to the session manager and create a session for this
     * thread
     */
    public void run()
    {
        for (;;)
        {
            try
            {
                // acquire a session
                session_ = PMDSessionManager.instance().getSession();
                stream_.writeVerbose("SessionAbortClient Created the session " + session_.toString());
    
                // bind a version of the vector
                session_.getCurrentTransaction().begin();
                VectorIfc vector = session_.getCollectionManager().createVector();
                session_.bind(("multivector" + Integer.toString(instance_++)), 
                    session_.getCollectionManager().createVector());
                stream_.writeVerbose("bound 1st object");
                session_.bind(("multivector" + Integer.toString(instance_++)), 
                    session_.getCollectionManager().createVector());
                stream_.writeVerbose("bound 2nd object");
                session_.bind(("multivector" + Integer.toString(instance_++)), 
                    session_.getCollectionManager().createVector());
                stream_.writeVerbose("bound 3rd and ready to abort");
                session_.getCurrentTransaction().abort();
    
                PMDSessionManager.instance().destroySession();
                stream_.writeVerbose("Destroyed the session " + session_.toString());
                break;
            }
            catch (Exception exception)
            {
                try
                {
                    session_.getCurrentTransaction().abort();
                    stream_.writeVerbose("SessionAbortClient exception " + exception);
                    Thread.sleep((long)(Math.random() * 100));
                }
                catch (Exception other)
                {
                    // ignore it
                }
            }
        }
    }

    /**
     * Return the session
     *
     * @return      SessionIfc
     */
    public SessionIfc getSession()
    {
        return session_;
    }


    private SessionIfc session_ = null;
    private CWVerboseStream stream_ = null;
    private int instance_ = 0;

    private static int count__ = 0;
}


class PMDInteger 
    extends PersistentObject
    implements Externalizable
{
    public PMDInteger()
    {
    }

    public PMDInteger(int value)
    {
        value_ = value;   
    }

	public int get()
	{
		return value_;
	}
	
    public String toString()
    {
        return Integer.toString(value_);
    }

    // implementation of Externalizable.writeExternal
    public void writeExternal(ObjectOutput stream)
        throws IOException
    {
        stream.writeInt(value_);
    }

    // implementation of Externalizable.writeExternal
    public void readExternal(ObjectInput stream)
        throws IOException, ClassNotFoundException
    {
        value_ = stream.readInt();
    }

    private int value_;
}



