/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2002-2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: TestChannelListener.java,v 1.3 2003/06/09 06:28:44 tanderson Exp $
 */
package org.exolab.core.test.messenger;

import java.io.IOException;
import java.io.Serializable;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.exolab.core.messenger.Channel;
import org.exolab.core.messenger.ChannelHandler;
import org.exolab.core.messenger.ChannelListener;


/**
 * Implements {@link ChannelHandler} and {@link ChannelListener}
 * for testing purposes
 *
 * @author <a href="mailto:tma@netspace.net.au">Tim Anderson</a>
 * @version $Revision: 1.3 $
 */
class TestChannelListener implements ChannelHandler, ChannelListener {

    /**
     * The set of opened channels
     */
    private HashSet _channels = new HashSet();

    /**
     * The number of times a channel is opened
     */
    private int _opened = 0;

    /**
     * The number of times a channel is closed
     */
    private int _closed = 0;

    /**
     * The logger
     */
    private static final Log _log = 
        LogFactory.getLog(TestChannelListener.class);


    /**
     * Invoked when a channel is opened
     *
     * @param channel the new channel
     */
    public synchronized void opened(Channel channel) {
        ++_opened;
        if (!_channels.contains(channel)) {
            _channels.add(channel);
        } else {
            _log.debug("received duplicate opened event");
        }
    }

    /**
     * Invoked when a channel is closed
     */
    public void closed() {
        ++_closed;
    }

    /**
     * Returns the number of times that a channel open event has been 
     * received
     */
    public synchronized int getOpened() {
        return _opened;
    }

    /**
     * Returns the number of times that a channel close event has been 
     * received
     */
    public synchronized int getClosed() {
        return _closed;
    }

    public void receive(Object request) {
    }

    public Object invoke(Object request) {
        return request;
    }
    
} //-- TestChannelListener
