/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 2002-2003 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: TestConnectionListener.java,v 1.3 2003/06/09 06:28:44 tanderson Exp $
 */
package org.exolab.core.test.messenger;

import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.exolab.core.messenger.Connection;
import org.exolab.core.messenger.ConnectionEventListener;
import org.exolab.core.messenger.ConnectionHandler;


/**
 * Implements {@link ConnectionHandler} and {@link ConnectionEventListener}
 * for testing purposes
 *
 * @author <a href="mailto:tma@netspace.net.au">Tim Anderson</a>
 * @version $Revision: 1.3 $
 */
class TestConnectionListener implements ConnectionEventListener, 
                                        ConnectionHandler {

    /**
     * The set of opened connections
     */
    private HashSet _connections = new HashSet();

    /**
     * The number of times a connected event is received
     */
    private int _opened = 0;

    /**
     * The number of times a close event is received
     */
    private int _closed = 0;

    /**
     * The number of times an error event is received
     */
    private int _errors = 0;
    
    /**
     * The logger
     */
    private static final Log _log = 
        LogFactory.getLog(TestConnectionListener.class);

    /**
     * Invoked when the connection is opened
     *
     * @param connection the new connection
     */
    public synchronized void connected(Connection connection) {
        ++_opened;
        if (!_connections.contains(connection)) {
            _connections.add(connection);
        } else {
            _log.debug("received duplicate connected event");
        }
    }

    /**
     * Invoked when a connection is closed
     *
     * @param connection the closed connection
     */
    public void closed(Connection connection) {
        ++_closed;
    }

    /**
     * Invoked when a fatal connection error occurs, just before an IOException
     * is thrown to the application
     *
     * @param connection the connection the error occurred on
     * @param exception the error
    */
    public synchronized void error(Connection connection, 
                                   Exception exception) {
        ++_errors;
        _log.error("Error on connection", exception);
    }

    /**
     * Returns the number of times that a connection open event has been 
     * received
     */
    public synchronized int getOpened() {
        return _opened;
    }

    /**
     * Returns the number of times that a connection close event has been 
     * received
     */
    public synchronized int getClosed() {
        return _closed;
    }

    /**
     * Returns the number of times that a connection error event has been 
     * received
     */
    public synchronized int getErrors() {
        return _errors;
    }
    
} //-- TestConnectionListener


